/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide('relm.app.Permissions'); //$NON-NLS-1$

(function(){

var permissions_ = null;

/**
 * Client side cache of the current user's security permissions, that is,
 * allowed operations and actions. This is only useful for enabling and disabling
 * controls in the web UI; security permissions are still checked server side during
 * each call.
 */
dojo.mixin(relm.app.Permissions, {

	/*
	 * Well known operation "categories" for RELM.
	 */
	VIEWS: 'com.ibm.relm.process.operations.view', //$NON-NLS-1$
	QUERIES: 'com.ibm.relm.process.operations.query', //$NON-NLS-1$
	PRODUCTS: 'com.ibm.relm.process.operations.product', //$NON-NLS-1$

	/*
	 * Well known actions for RELM.
	 */
	CREATE: 'create', //$NON-NLS-1$
	DELETE: 'delete', //$NON-NLS-1$
	MODIFY: 'modify', //$NON-NLS-1$
	CHANGE_PARAMETERS: 'changeParameters', //$NON-NLS-1$
	VIEW: 'view', //$NON-NLS-1$
	ADD_PROPERTY: 'addProperty', //$NON-NLS-1$
	DELETE_PROPERTY: 'deleteProperty', //$NON-NLS-1$
	BRANCH: 'branch', //$NON-NLS-1$
	CREATE_SNAPSHOT: 'createSnapshot', //$NON-NLS-1$
	REDACT: 'redact', //$NON-NLS-1$

	/**
	 * Initializes the global cache of permissions for the current user.
	 * Must be called during start-up and after each new log in.
	 * Assumes the user is logged in to a single project area on this page.
	 */
	initForCurrentUser: function() {
		
		var deferred = new dojo.Deferred();

		jazz.client.xhrGet({
			url: net.jazz.ajax._contextRoot + '/permissions', //$NON-NLS-1$
			handleAs: 'json', //$NON-NLS-1$
		
			load: function (permissions) {
				console.debug('Initialized user permissions. Log out or refresh this page to see any permission changes.'); //$NON-NLS-1$
				permissions_ = permissions;
				deferred.callback();
			},
		
			error: function(error, ioArgs) {
				var errorObject = dojo.fromJson(ioArgs.xhr.responseText);
				deferred.errback(errorObject);
			}
		});

		return deferred;
	},

	/**
	 * Checks if the authenticated user is a Jazz Admin
	 * @return {Boolean} true indicates the user is a jazz admin
	 */
	isJazzAdmin: function() {
		var roles = dojo.getObject('com.ibm.team.repository.web.client.internal.AUTHENTICATED_CONTRIBUTOR_ROLES'); //$NON-NLS-1$
		var isJazzAdmin = dojo.indexOf(roles, 'JazzAdmins') !== -1; //$NON-NLS-1$
		return isJazzAdmin;
	},

	/**
	 * Checks if a user can perform an
	 * operation and action, that is, if they have that permission.
	 * @param  {string} operation name of operation category.
	 * @param  {string} action    name of action.
	 * @return {boolean}           true if they have permission.
	 */
	can: function(operation, action) {
		// This shouldn't happen as Permissions are initialized during login (RelmApplication.start)
		if (!permissions_) {
			console.warn('Permissions have not been loaded for the current user. You may not have selected a project area yet.'); //$NON-NLS-1$
			console.warn('Assuming the operation is allowed. Server side security will prevent the operation if not allowed: ' + operation + ' - '+ action); //$NON-NLS-1$ //$NON-NLS-2$
			return true;
		}

		var actions = permissions_[operation];
		if (!actions) return false;
		return actions[action];
	},

	canProductsModify: function() {
		return this.can(this.PRODUCTS, this.MODIFY);
	},

	canProductsBranch: function() {
		return this.can(this.PRODUCTS, this.BRANCH);
	},

	canProductsCreateSnapshot: function() {
		return this.can(this.PRODUCTS, this.CREATE_SNAPSHOT);
	},

	canProductsAddProperty: function() {
		return this.can(this.PRODUCTS, this.ADD_PROPERTY);
	},

	canProductsDeleteProperty: function() {
		return this.can(this.PRODUCTS, this.DELETE_PROPERTY);
	},

	canViewsCreate: function() {
		return this.can(this.VIEWS, this.CREATE);
	},

	canProductsRedact: function() {
		return this.can(this.PRODUCTS, this.REDACT);
	}
});

})();
