/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.common.web.ResourceLinkBatchLoader'); //$NON-NLS-1$

(function(){

	/**
	 * This matches the default "small" page size of search results so that a
	 * full page can load at once. It's also reasonably fast most of the time.
	 */
	var BATCH_SIZE = 25;

	/**
	 * Handles a stack of jazz.ui.ResourceLinks that need to be loaded to show
	 * OSLC previews titles and icons. This is done with resourceLink.refresh.
	 * Links are loaded in reasonable size batches so as to not make any
	 * single request that takes too long. For example, if the use disables
	 * paging in search or query results, there can be 100s or thousands of
	 * links. Loading all these at once can take several minutes with slow
	 * servers. Instead of waiting for all links, this will get a few at a
	 * times and load links from the top to bottom. This both makes the first
	 * links load faster, and also lets user still get rich hovers on lower
	 * links if they want (before all have loaded) since their request will
	 * will get a chance to run between batches.
	 */
	dojo.declare(
		'com.ibm.relm.common.web.ResourceLinkBatchLoader', //$NON-NLS-1$
		[],
		{
			constructor: function()
			{
				this._remainingLinks = [];	
				this._loadTimeout = null;
				this._pendingLoadCount = 0;
			},

			/**
			 * Adds a links to request. It will be loaded soon, but not
			 * immediately. You can add as many links at a time this way and
			 * they will be intelligently loaded in batches as soon as
			 * possible.
			 * @param {jazz.ui.ResourceLink} jazzLink link to load.
			 */
			add: function(jazzLink) {
				this._remainingLinks.push(jazzLink);
				this._schedule();
			},

			/**
			 * Schedules a batch of links to load. If there are already
			 * pending requests, does nothing, knowing that they will be
			 * loaded when the current request finished anyway.
			 */
			_schedule: function() {
				if (this._loadTimeout) return;
				if (this._remainingLinks.length === 0) return;

				var self = this;
				this._loadTimeout = setTimeout(function() {
					self._loadNextBatch();
				}, 0);
			},

			/**
			 * Called by _schedule to load the next batch of links.
			 * No-op of there are already pending loads (which will do
			 * another load once completed if needed).
			 */
			_loadNextBatch: function() {
				delete this._loadTimeout;

				if (this._pendingLoadCount > 0) return;

				// Extract the earliest links first
				var all = this._remainingLinks;
				var batch = all.slice(0, BATCH_SIZE);
				all.splice(0, batch.length);

				var self = this;
				for (var i = 0; i < batch.length; i++) {
					var jazzLink = batch[i];

					// Register for callback from the loaded response. This
					// indicates we should schedule the next batch.
					jazzLink.connect(jazzLink, '_batchedPresentationHandler', function(result) { //$NON-NLS-1$
						self._pendingLoadCount = Math.max(0, self._pendingLoadCount - 1);
						self._schedule();
					});
					jazzLink.connect(jazzLink, '_errorRefresh', function(result) { //$NON-NLS-1$
						self._pendingLoadCount = Math.max(0, self._pendingLoadCount - 1);
						self._schedule();
					});

					// Cause the actual load of the link. The underlying link
					// already knows how to batch this load
					// with other links
					jazzLink.refresh();
					this._pendingLoadCount++;
				}
			},

			/**
			 * Dumps the current set of links to load and stops any further
			 * bath requests for that set. You can now start adding new links
			 * to be loaded. This is used when a page is unloaded, results are cleared,
			 * etc., but its simpler to resuse this isntance than create a new one.	
			 */
			reset: function() {
				clearTimeout(this._loadTimeout);
				delete this._loadTimeout;
				this._pendingLoadCount = 0;
				this._remainingLinks = [];
			}
		}
	);

})();
