/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.common.web.ResourceLinkScrollLoader'); //$NON-NLS-1$

dojo.require('dijit.dijit'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ResourceLinkBatchLoader'); //$NON-NLS-1$

(function(){
	function getMaxY(position) {
		return position.t + position.h;
	}

	/**
	 * Loads resource link previews as a user scrolls the page or resizes the window.
	 * This is to prevent loading too many previews at once when the user can't see them
	 * anyway. This isn't perfect, since the titles that aren't loaded yet can't be
	 * filtered or searched for with ctrl-F, but it's much faster.
	 *
	 * WARNING: This assumes all resource links are in a vertical list: they must be added in their
	 * page position from top to bottom.
	 */
	dojo.declare(
		'com.ibm.relm.common.web.ResourceLinkScrollLoader', //$NON-NLS-1$
		[],
		{
			/**
			 * Constructs a new scroll loader. This can be resued if reset later.
			 * @param  {[type]com.ibm.relm.common.web.ResourceLinkBatchLoader} resourceLinkBatchLoader a batch loader
			 * used to actually to the loading when needed.
			 */
			constructor: function(resourceLinkBatchLoader)
			{
				this._resourceLinks = [];
				this._resourceLinkRecords = [];
				this._batchLoader = resourceLinkBatchLoader;

			},

			/**
			 * Add a resource link here to watch and load it when in view.
			 * Must call .start will to start loading. Links must be added in their page position from top to bottom.
			 * @param  {ResourceLink} jazzLink the link to lazy-load.
			 */
			watchResourceLink: function(jazzLink) {
				this._resourceLinks.push(jazzLink);
				this._resourceLinkRecords.push(null);
			},

			/**
			 * Call this after all resource links are added to start
			 * loading visible links.	
			 */
			start: function() {
				// if (this._firstPosition) throw new Error('Trying to load resource links on scroll without a reset.'); //$NON-NLS-1$

				this._scrollHandle = dojo.connect(window, 'onscroll', this, this._onScroll); //$NON-NLS-1$
				this._resizeHandle = dojo.connect(window, 'onresize', this, this._onResize); //$NON-NLS-1$

				this._loadVisibleLinks();
			},

			/**
			 * If the link position have changed on a page due to hiding and showing nodes
			 * on the page (e.g., after running a text filter) you must call this since
			 * it's not auto detected and new link may be visible.
			 * @return {[type]} [description]
			 */
			positionsDidChange: function() {
				for (var i = 0; i < this._resourceLinkRecords.length; i++) {
					var record = this._resourceLinkRecords[i];
					if (record) delete record.position;
				}

				this._setNeedsLoad();
			},

			/**
			 * Gets a link record and caches the link's position on the page
			 * if not already done.
			 * @param  {number} i index of record to get.
			 * @return {object}   object with .position property guaranteed.
			 */
			_getRecordWithPosition: function(i) {
				var record = this._getRecord(i);

				if (!record.position) {
					record.position = dojo.position(this._resourceLinks[i].domNode, true);
					if (record.position.w === 0|| record.position.h === 0) {
						// The node was probably hidden with display: none. This isn't a valid position.
						delete record.position;
						return null;
					}
				}

				return record;
			},

			/**
			 * Gets a possibly empty, but never null record of cached information about
			 * the link at the same index.
			 * @param  {number} i record index.
			 * @return {object}   object with possibly cached details about the link.
			 */
			_getRecord: function(i) {
				var record = this._resourceLinkRecords[i];

				if (!record) {
					record = this._resourceLinkRecords[i] = {};
				} 

				return record;
			},

			/**
			 * Runs a binary search for the ResourceLink on the page
			 * that's closest to the given Y position.
			 * Assumes links are in order from top to bottom on the page
			 * so that each link is "lower" than the next.
			 * This is done since dojo.position can be very slow, so this
			 * reduces the number of lookup needed.
			 * 		
			 * @param  {number} desiredY the Y position we want to find a link at.
			 * @return {number}          the index of the link in the _resourceLinks array.
			 */
			_findLinkIndexForY: function(desiredY) {
				var min = 0;
				var midpoint = 0;
				var max = this._resourceLinkRecords.length - 1;

				while (min <= max) {
					midpoint = Math.round((max - min) / 2) + min;
					
					var record = this._getRecordWithPosition(midpoint);
					if (!record) {
						// This isn't really right, but we have no idea where we are now
						// since this link must be hidden. Need to make progress though
						min += 1;
						continue;
					}

					var diffX = record.position.y - desiredY;

					if (Math.abs(diffX) < 20) {
						return midpoint;
					}
					else if (diffX > 0) {
						max = midpoint - 1;
					}
					else {
						min = midpoint + 1;
					}
				}

				// console.debug('Could not found resource link near desired Y position. Using item ' + midpoint + ' at ' + record.position.y); //$NON-NLS-1$ //$NON-NLS-2$
				return midpoint;
			},

			/**
			 * Loads a link unless already loaded.
			 * @param  {number} i index of link to load.
			 */
			_loadLinkAt: function(i) {
				var resourceLink = this._resourceLinks[i];
				var record = this._getRecord(i);

				if (record.isLoaded) {
					return;
				}

				record.isLoaded = true;
				// resourceLink.domNode.style.backgroundColor = 'yellow'; //$NON-NLS-1$
				this._batchLoader.add(resourceLink);
			},

			/**
			 * Loads all link in the viewport.
			 */
			_loadVisibleLinks: function() {
				var viewport = dijit.getViewport();
				var topY = viewport.t;
				var bottomY = getMaxY(viewport);

				var topIndex = this._findLinkIndexForY(topY);
				var bottomIndex = this._findLinkIndexForY(bottomY);

				var visibleCount = bottomIndex - topIndex;
				
				// Add a full extra page of links to prevent "pop in" of links
				bottomIndex += visibleCount;

				for (var i = topIndex; i <= bottomIndex; i++) {
					if (i >= this._resourceLinks.length) break;

					this._loadLinkAt(i);
				}
			},

			/**
			 * Will load links after a short delay. Used to prevent too many
			 * loads while scrolling: you need to stop a bit before a load happens.
			 */
			_setNeedsLoad: function() {
				clearTimeout(this._loadTimeout);

				var self = this;
				this._loadTimeout = setTimeout(function() {
					self._loadVisibleLinks();
				}, 200);
			},

			_onScroll: function() {
				this._setNeedsLoad();
			},

			_onResize: function() {
				this.positionsDidChange();
			},

			/**
			 * Resets this to be used with new links.
			 * You must watch new ones, then call .start again.
			 */
			stopAndReset: function() {
				dojo.disconnect(this._scrollHandle);
				dojo.disconnect(this._resizeHandle);

				this._resourceLinks = [];
				this._resourceLinkRecords = [];
				this._batchLoader.reset();
			}
		}
	);

})();