/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide('com.ibm.relm.common.web.enums.EnumProperties'); //$NON-NLS-1$

dojo.require('com.ibm.relm.common.web.enums.EnumStore'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.enums.EnumValueRow'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.RequiredMarker'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.Section'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.table.HoverRow'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('dijit._Widget'); //$NON-NLS-1$
dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.require('jazz.ui.StyledTable'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.common.web.enums', 'EnumManagerDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$

(function(){

	/**
	 * EnumProperties is a widget for displaying and editing the properties
	 * of an enumeration, namely its name (label) and enumeration values (members).
	 * 
	 * Enumeration values can be added, deleted, ordered, and sorted.
	 */
	dojo.declare(
		'com.ibm.relm.common.web.enums.EnumProperties', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.common.web','enums/templates/EnumProperties.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,
			messages: dojo.i18n.getLocalization('com.ibm.relm.common.web.enums', 'EnumManagerDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$
			enumeration: null, // Provided in the constructor
			_table: null, // the table of EnumValueRows
			_nameInput: '', //$NON-NLS-1$
			
			/**
			 * 
			 * @param args.enumeration The required enum object:
			 * 		{uri, label, type, members: [{uri, label, substitution}, ...]}
			 * @param node Optional node to replace.
			 */
			constructor: function(args, node) {
				this._tableDivId = dijit.getUniqueId('tableDivId');//$NON-NLS-1$
			},

			postCreate: function() {
				this.inherited(arguments);
				this._widgets = [];
				
				this._originalName = this._previousName = this.enumeration.label; // used to compare against subsequent changes
				this._nameInput.value = this.enumeration.label;
				this._valuesLabel = dojo.query('.relm-Section-title',this._valuesSection.domNode)[0]; //$NON-NLS-1$
				dojo.attr(this._valuesLabel,'for',this._tableDivId);//$NON-NLS-1$

				// The 'Add' link for new members
				var addLink = dojo.create('a', { //$NON-NLS-1$
					href: '#', //$NON-NLS-1$
					title: this.messages.addEnumValueButtonTooltip,
					innerHTML: this.messages.addEnumValueButton,
					className: 'relm-EnumProperties-sectionLink' //$NON-NLS-1$
				});
				this.connect(addLink, 'onclick', dojo.hitch(this, function(e) { //$NON-NLS-1$
					dojo.stopEvent(e);
					this.addEnumValue(e);
				}));
				
				this._valuesSection.setControls(addLink);
				
				var args = {
					columns: [
						{label: this.messages.nameColumnHeading, sortable: true},
						{label: this.messages.substitutionColumnHeading, sortable: true},
						{label: this.messages.actionColumnHeading, className: 'relm-EnumProperties-tableToolbarCol'} //$NON-NLS-1$
					],
					handleSort: dojo.hitch(this, this._handleSort)
				};
				
				this._table = new jazz.ui.StyledTable(args);
				this._widgets.push(this._table);
				dojo.addClass(this._table.domNode, 'relm-EnumProperties-table'); //$NON-NLS-1$
				this._setEnumValues(this.enumeration.members);
				this._table.placeAt(this._enumProperties);
				this._table.startup();
			},
			
			/**
			 * Gets the enumeration values (members) from the table and returns them as an array:
			 * [{uri, label, substitution}, ...]
			 * Skips over ones that are both new and empty.
			 */
			_getEnumValues: function() {
				var enumValues = [];
				
				dojo.forEach(this._table.tbody.rows, function(row){
					var enumValueWidget = dijit.getEnclosingWidget(row);
					if (!enumValueWidget.isNewAndEmpty()) {
						enumValues.push(enumValueWidget.getCurrentEnumValue());	
					}
				});
				
				return enumValues;
			},
			 
			_setEnumValues: function(enumValues) {
				this._table.clearBody();
				this._buildRows(enumValues);
			},
			
			/**
			 * Check for the following:
			 * - At least one member is defined
			 * - All members have unique names
			 * - All members have names
			 * Empty rows are ignored.
			 * 
			 * @param {array} The enumeration values.
			 * @return true if the enum properties validate
			 * @throws exception if any of the above criteria are not met
			 */
			_validate: function(enumValues) {
				var enumName = this._nameInput.value;
				
				if (enumValues.length == 0) {
					throw new Error(dojo.replace(this.messages.localEnumHasNoValues, [enumName]));
				}
				
				var names = {};
				
				dojo.forEach(enumValues, function(enumValue){
					var enumValueName = enumValue.label;
					
					if (!enumValueName) {
						throw new Error(dojo.replace(this.messages.enumValueWithoutName, [enumName]));
					}
					
					if (names[enumValueName]) {
						throw new Error(dojo.replace(this.messages.enumWithDuplicateValues, [enumName, enumValueName]));
					} else {
						names[enumValueName] = true;
					}
				}, this);
				
				return true;
			},
			
			/**
			 * Returns an enum object reflecting the current state of the widget
			 * if and only if it differs from the enum used to create it in the
			 * first place. Otherwise, returns null.
			 * 
			 * @throws exception if validation of the data fails
			 */
			getCurrentRepresentationIfDirty: function() {
				var currentEnum = {
					uri: this.enumeration.uri,
					label: this._nameInput.value,
					type: this.enumeration.type,
					members: this._getEnumValues()
				};
				
				this._validate(currentEnum.members);
				
				// new enums are always dirty
				if (com.ibm.relm.common.web.enums.EnumStore.uriIsNew(currentEnum.uri)) {
					return currentEnum;
				}
				
				// only the label and members could have changed
				return ((currentEnum.label == this._originalName)
					&& this._membersMatch(currentEnum.members, this.enumeration.members))
					? null
					: currentEnum;
			},
			
			/**
			 * Test to see if the given lists of enumeration members are equal.
			 */
			_membersMatch: function(members1, members2) {
				if (members1.length == members2.length) {
					for (var i = 0; i < members1.length; i++) {
						var m1 = members1[i];
						var m2 = members2[i];
						
						if (!((m1.uri == m2.uri)
							&& (m1.label == m2.label)
							&& (m1.substitution == m2.substitution))) {
							return false;
						}
					}
					return true;
				} else {
					return false;
				}
			},

			_buildRows: function(enumValues) {
				dojo.forEach(enumValues, function(enumValue){
					var row = new com.ibm.relm.common.web.enums.EnumValueRow({ enumValue: enumValue });
					this._widgets.push(row);
					row.placeAt(this._table.tbody);
					row.startup();
				}, this);
			},
			
			addEnumValue: function(e) {
				var blankEnumValue = { uri: '', label: '', substitution: '' }; //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				var row = new com.ibm.relm.common.web.enums.EnumValueRow({ enumValue: blankEnumValue });
				this._widgets.push(row);
				row.placeAt(this._table.tbody);
				row.startup();
				row.focus();
				com.ibm.relm.common.web.ui.util.flashNode(row.domNode);
			},
			
			_onNameBlur: function(e) {
				var name = this._nameInput.value = dojo.trim(this._nameInput.value);
				
				// A name is required--restore it to the last one
				if (!name) {
					this._nameInput.value = this._previousName ;
				} else if (name != this._previousName) {
					this._previousName = name;
					this.onNameChange(this.enumeration.uri, name);
				}
			},
			
			/**
			 * Handler for when the enum name changes.
			 * This function is available to be overridden by callers or used with dojo.connect.
			 * 
			 * @param {string} enumUri
			 * @param {string} newName
			 */
			onNameChange: function(enumUri, newName) {
				// no-op
			},
			
			selectName: function() {
				this._nameInput.select();
			},
			
			_handleSort: function() {
				var sortBy = this._table.get('sortBy'); //$NON-NLS-1$
				var ascending = this._table.get('sortByAscending'); //$NON-NLS-1$
				
				var comparator = (sortBy == 0
					? function(a, b) { return _baseComparator(a.label, b.label, ascending); }
					: function(a, b) { return _baseComparator(a.substitution, b.substitution, ascending); });
				
				var enumValues = this._getEnumValues();
				enumValues.sort(comparator);
				this._setEnumValues(enumValues);
			},
			
			uninitialize: function() {
				// Some rows may already be deleted through the row itself,
				// but calling destroyRecursive doesn't appear problematic.
				console.debug('Destroying EnumProperties'); //$NON-NLS-1$
				for (var i = 0; i < this._widgets.length; i++) {
					var w = this._widgets[i];
					w.destroyRecursive();
				}
			},
		}); // declare

		function _baseComparator(a, b, ascending) {
			var result = function() {
				if (!a) {
					return !b ? 0 : -1;
				}
				if (!b) {
					return 1;
				}
				
				var aLower = a.toLowerCase();
				var bLower = b.toLowerCase();
				
				if (aLower < bLower) {
					return -1;
				}
				if (aLower > bLower) {
					return 1;
				}
				
				return 0;
			}();
			
			return ascending ? result : result * -1;
		}
	})();