/******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2013. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide('com.ibm.relm.common.web.enums.EnumStore'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.common.web.enums', 'EnumManagerDialogMessages'); //$NON-NLS-1$ //$NON-NLS-2$

// Be careful not to use the identifier 'enum'; it's a keyword in JavaScript!

(function() {
	
	/**
	 * A store for enumerations, which may be referenced when defining query
	 * parameters. Enums are conceptually just lists.
	 * 
	 * Representation:
	 * [{uri, label, type, members: [{uri, label, substitution}, ... ]}, ...]
	 * where type is 'local'|'external'.
	 */
	dojo.declare('com.ibm.relm.common.web.enums.EnumStore', null, { //$NON-NLS-1$
		
		messages: dojo.i18n.getLocalization('com.ibm.relm.common.web.enums', 'EnumManagerDialogMessages'), //$NON-NLS-1$ //$NON-NLS-2$
		
		/**
		 * Create a new, empty store.
		 * 
		 * @param enums Optional enumerations with which to create the store.
		 */
		constructor: function(enums) {
			this._enums = enums ? enums : [];
			this._sort();
		},
		
		/**
		 * Load the store from the server.
		 *
		 * @return {Deferred} The enums are passed to the callback. The errback
		 * 		gets the JSON object from the server exception.
		 */
		load: function() {
			var deferred = new dojo.Deferred();
			
			jazz.client.xhrGet({
				url: net.jazz.ajax._contextRoot + '/ple/enum', //$NON-NLS-1$
				handleAs: 'json', //$NON-NLS-1$
				failOk: true,
				
				load: dojo.hitch(this, function(results) {
					this._enums = results;
					this._sort();
					deferred.callback(results);
				}),
				
				error: function(error, ioArgs) {
					console.error('Failed to load enumerations.', ioArgs); //$NON-NLS-1$
					deferred.errback(dojo.fromJson(ioArgs.xhr.responseText));
				}
			});

			return deferred;
		},
		
		/**
		 * Return the enum labels (names) in case-insensitive alphabetically ascending order.
		 * 
		 * @return {array} The enum labels in case-insensitive alphabetically ascending order.
		 */
		getEnumLabels: function() {
			var labels = [];
			
			for (var i = 0; i < this._enums.length; i++) {
				labels.push(this._enums[i].label);
			}
			
			return labels;
		},
		
		/**
		 * Return the raw enums in case-insensitive alphabetically ascending
		 * order.
		 * 
		 * @return {array} The raw enum array in case-insensitive
		 * 		alphabetically ascending order.
		 *
		 */
		getEnums: function() {
			return this._enums;
		},
		
		/**
		 * Get the raw enum for the given URI.
		 * 
		 * @param {string} uri The enum URI.
		 * @return {enumObject} The raw enum, or null if it isn't found.
		 */
		getEnum: function(uri) {
			for (var i = 0; i < this._enums.length; i++) {
				var enumeration = this._enums[i];
				if (enumeration.uri == uri)
					return enumeration;
			}
			
			return null;
		},

		_getEnumIndex: function(uri) {
			for (var i = 0; i < this._enums.length; i++) {
				var enumeration = this._enums[i];
				if (enumeration.uri == uri)
					return i;
			}
			
			throw new Error('_getEnumIndex: Enum not found: ' + uri); //$NON-NLS-1$
		},
		
		/**
		 * Add a new internal enum with the given label.
		 * 
		 * @param {string} label The label of the enum.
		 * @return {enumObject} The raw enum. The URI field is temporary until
		 * 		the store is saved.
		 */
		addLocalEnum: function(label) {
			var uri = EnumStore.generateUniqueTempUri();
			var enumeration = {uri: uri, label: label, type: 'local', members: []}; //$NON-NLS-1$
			this._enums.push(enumeration);
			this._sort();
			return enumeration;
		},
		
		/**
		 * Add an externally defined enum by reference, provided its
		 * details can be retrieved. Only the URI is persisted; the other properties are looked up.
		 * 
		 * @param {string} uri The uri of an external enum.
		 * @return {Deferred} The external enum is passed to the callback.
		 * 		The errback gets the JSON object from the server exception.
		 */
		addExternalEnum: function(uri) {
			var deferred = new dojo.Deferred();
			
			jazz.client.xhrGet({
				url: net.jazz.ajax._contextRoot + '/ple/enum', //$NON-NLS-1$
				handleAs: 'json', //$NON-NLS-1$
				failOk: true,
				content: {
					uri: uri,
					enumType: "external" //$NON-NLS-1$
				},
				
				load: dojo.hitch(this, function(enumeration) {
					if (enumeration.members.length == 0) { // no members, don't add
						deferred.errback({
							type: jazz.ui.MessageBox.STATUS_TYPES.WANRING,
							summary: dojo.replace(this.messages.externalEnumHasNoValues, [enumeration.label, uri]),
						});
					} else { // has members
						this._enums.push(enumeration);
						this._sort();
						deferred.callback(enumeration);		
					}
				}),
				
				error: function(error, ioArgs) {
					console.error('Failed to load external enumeration.', ioArgs); //$NON-NLS-1$
					deferred.errback(dojo.fromJson(ioArgs.xhr.responseText));
				}
			});

			return deferred;
		},
		
		/**
		 * Give the enum a new label.
		 * 
		 * @param {string} uri The enum URI.
		 * @param {string} newlabel The new label to assign.
		 * @return {string} The old (replaced) label.
		 */
		relabelEnum: function(uri, newlabel) {
			var enumeration = this.getEnum(uri);
			var oldlabel = enumeration.label;
			enumeration.label = newlabel;
			this._sort();
			return oldlabel;
		},
		
		/**
		 * Delete an enum.
		 * 
		 * @param {string} uri The enum URI.
		 * @return {enumObject} The removed enum.
		 */
		removeEnum: function(uri) {
			var index = this._getEnumIndex(uri);
			var enumeration = this._enums[index];
			this._enums.splice(index, 1);
			return enumeration;
		},
		
		_sort: function() {
			this._enums.sort(function(a, b){
				return _baseComparator(a.label, b.label);
			});	
		},
		
		serialize: function() {
			return dojo.toJson(this._enums);
		}
	});	// dojo.declare
	
	var EnumStore = com.ibm.relm.common.web.enums.EnumStore;
	
	EnumStore.NEW_URI_PREFIX = 'new:'; //$NON-NLS-1$
	
	EnumStore.generateUniqueTempUri = function() {
		return EnumStore.NEW_URI_PREFIX + dijit.getUniqueId('com.ibm.relm.common.web.enums.EnumStore'); //$NON-NLS-1$
	};
	
	EnumStore.uriIsNew = function(uri) {
		return uri.indexOf(EnumStore.NEW_URI_PREFIX) == 0;
	};
	
	EnumStore.newEnumMember = function(label, substitution) {
		if (arguments.length == 0) {
			label = substitution = ''; //$NON-NLS-1$
		} else if (arguments.length == 1) {
			label = ''; //$NON-NLS-1$
		}
		return { uri: EnumStore.generateUniqueTempUri(), label: label, substitution: substitution };
	};

	/**
	 * Compare two strings case-insensitively, accounting for nulls.
	 */
	function _baseComparator(a, b) {
		if (!a) {
			return !b ? 0 : -1;
		}
		if (!b) {
			return 1;
		}
		
		var aLower = a.toLowerCase();
		var bLower = b.toLowerCase();
		
		if (aLower < bLower) {
			return -1;
		}
		if (aLower > bLower) {
			return 1;
		}
		
		return 0;
	}
})();