/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/

dojo.provide("com.ibm.relm.explorer.web.explorer.queryparams.SparqlCommentStripper"); //$NON-NLS-1$

(function(){

	/**
	 * Given a SPARQL string, removes all comments from the string.
	 * This is needed during parameter substitution to ensure commented
	 * out parameters are ignored.
	 * <p>
	 * Understands strings and IRIs in SPARQL so that embedded "#" won't be
	 * treated as comments.
	 * <p>
	 * 
	 * @since 5.0.1
	 */
	dojo.declare(
		"com.ibm.relm.explorer.web.explorer.queryparams.SparqlCommentStripper", //$NON-NLS-1$
		[ dijit._Widget ],
		{
			/**
			 * Original, commented SPARQL.
			 */
			// before: ,
			
			/**
			 * An array of the letters in the original, commented SPARQL.
			 * Used to iterate over.
			 */
			// chars: ,
			
			/**
			 * The processed, comment-stripped SPARQL.
			 * Starts as empty array. As we walk through the original 
			 * SPARQL, all non-comment chars are added here.
			 * <p/>
			 * Call .join("") to get sparql string.
			 */
			// after: ,
			
			/**
			 * When tokenizing the before-SPARQL, this is the index of the next
			 * char to process.
			 * Starts at 0 and walks to the end as we progress.
			 */
			// nextCharIndex

			/**
			 * @param commnentedSparql
			 * 		the SPARQL to strip comments out it. May not actually contain comments.
			 */
			constructor: function(sparql) {
				this.before = sparql;
				this.chars = sparql.split(""); //$NON-NLS-1$
				this.after = [];
				this.nextCharIndex = 0;
			},

			/**
			 * Strips the comments from the SPARQL this instance wraps.
			 * Comments are removed from their opening # to the end of the line.
			 * No other changes are made to the string.
			 * @return
			 * 		the SPARQL string with all comments removed.
			 */
			strip: function() {
				this.inDefaultState();
				return this.after.join(""); //$NON-NLS-1$
			},

			/**
			 * The default state of this tokenizer.
			 * Consumes on character at a time from the before-SPARQL.
			 * If the character does not mark a new token, it is added
			 * to the after-SPARQL. If it does start a new token, then
			 * we switch to that token state.
			 * <p>
			 * Note that the opening character
			 * for a token is always "consumed" by this loop, so when switching
			 * to another token state, that state is responsible for saving the
			 * opening character to the after-SPARQL if it should be preserved.
			 * <p>
			 * Other states always return here when done (there are no nested states).
			 */
			inDefaultState: function() {
				var c;
				
				while ((c = this.nextChar()) !== null) {
					switch (c) {
					case '"': //$NON-NLS-1$
					case '\'': //$NON-NLS-1$
						this.inString(c);
						break;
						
					case '<': //$NON-NLS-1$
						this.inIriState();
						break;
					
					case '#': //$NON-NLS-1$
						this.inCommentState();
						break;
						
					default:
						this.after.push(c);
						continue;
					}
				}
			},

			/**
			 * Consumes a string, either single or triple quoted.
			 * May contain # chars that don't mark a comment.
			 * 
			 * @param openingQuote
			 * 		the single or double-quote that starts the string. 
			 */
			inString: function(openingQuote) {
				if (openingQuote != "'" && openingQuote != '"') console.warn("Char parameter wasn't an opening quote"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
				
				if (this.isAtTripleQuote(openingQuote)) {
					this.skipChars(2);
					this.inTripleQuotedStringState(openingQuote);
				}
				else {
					this.inSingleQuotedStringState(openingQuote);
				}
			},
			
			inSingleQuotedStringState: function(openingQuote) {
				this.after.push(openingQuote);
				
				var c;
				while ((c = this.nextChar()) !== null) {
					this.after.push(c);
					
					switch (c) {
					case '\\': //$NON-NLS-1$
						// Skip escaped chars
						this.after.push(this.nextChar());
						break;
						
					case '\'': //$NON-NLS-1$
					case '"': //$NON-NLS-1$
						if (c !== openingQuote) continue;
						
						return;	
						
					case '\n': //$NON-NLS-1$
					case '\r': //$NON-NLS-1$
						// An invalid string. Newline without closing quote.
						// Exit anyway
						return;
					}				
				}
			},

			inTripleQuotedStringState: function(openingQuote) {
				this.after.push(openingQuote, openingQuote, openingQuote);
				
				var c;
				while ((c = this.nextChar()) !== null) {
					this.after.push(c);
					
					switch (c) {
					case '\\': //$NON-NLS-1$
						// Skip escaped chars. Are these the same for triple-quote strings?
						this.after.push(this.nextChar());
						break;
						
					case '\'': //$NON-NLS-1$
					case '"': //$NON-NLS-1$
						if (c !== openingQuote) continue;
						
						if (this.isAtTripleQuote(c)) {
							this.after.push(this.nextChar(), this.nextChar());
							return;
						}
					}				
				}
			},
			
			/**
			 * Consumes an IRI (URI).
			 */
			inIriState: function() {
				this.after.push('<'); //$NON-NLS-1$
				
				var c;
				while ((c = this.nextChar()) !== null) {
					this.after.push(c);
					
					switch (c) {
					case '>': //$NON-NLS-1$
						return;
						
					default:
						continue;
					}
				}
				return;
			},


			/**
			 * Read chars until we hit a new-line. Does not save them to the after-SPARQL--
			 * the comment is stripped.
			 */
			inCommentState: function() {
				var c;
				
				while ((c = this.nextChar()) !== null) {
					switch (c) {
					case '\r': //$NON-NLS-1$
					case '\n': //$NON-NLS-1$
						this.after.push(c);
						return;
						
					default:
						continue;
					}
				}
			},


			/**
			 * Checks if the quote char we're at it a triple quote (followed by two more of the same.
			 * 
			 * @param quoteChar
			 * 		the first (of possibly three) quote chars that we're currently
			 * 		at. The next two chars must match.
			 * @return
			 */
			isAtTripleQuote: function(quoteChar) {
				return quoteChar === this.peekChar(0) && quoteChar === this.peekChar(1);
			},

			skipChars: function(count) {
				for (var i = 0; i < count; i++) {
					this.nextChar();
				}
			},

			/**
			 * Gets the next char in the before-SPARQL to handle
			 * and increments the index.
			 * @return
			 * 		-1 if the end of the text was reached. Otherwise,
			 * 		the next character from the stream.
			 */
			nextChar: function() {
				var c = this.peekChar(0);
				if (c === null) return null;
				
				this.nextCharIndex++;
				return c;
			},
			
			peekChar: function(offset) {
				var peekIndex = this.nextCharIndex + offset;
				
				if (peekIndex >= this.chars.length) {
					return null;
				}
				
				return this.chars[peekIndex];
			}
		});

})();