/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.common.web.ui.AutoHeightBox'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.util'); //$NON-NLS-1$

(function(){
	var util = com.ibm.relm.common.web.ui.util;

	/**
	 * Listens to page resize events to auto-resize the height of elements to not exceed the
	 * height of the current jazz content area. This is done to allow some views to have local
	 * scrollbars, instead of forcing the whole page (and thus all content) to scroll. Notably,
	 * it's used for the PLE product tree.
	 * 
	 * Note that this must be constructed around a DOM node, which is what will be resized.
	 */
	dojo.declare(
		"com.ibm.relm.common.web.ui.AutoHeightBox", //$NON-NLS-1$
		null, 
		{

			/**
			 * @box the DOM node to resize. This node will be made position 'absolute'.
			 * to take it out of the normal flow (its height should not influence the content height).
			 * This may require special CSS rules to get it to show the right width, but in many cases the
			 * default width will work.
			 * @args.priority passed directly to jazz.util.ResizeNotifier
			 * @args.force  passed directly to jazz.util.ResizeNotifier
			 * @args.listenOnPage if a page ID is given, the resizes will only happen
			 * when the page with this ID is in focus. Just a performance optimization.
			 */
			constructor: function(box, args) 
			{
				args = args || {};
				this.box = box;
				this._priority = args.priority || 5;
				this._force = args.force === undefined ? true : args.force;
				this._fillParentWidth = args.fillParentWidth;

				// Take this out of normal flow so it can't influence the size of the content area.
				// May require some more precise positioning by the caller.
				this.box.style.position = 'absolute'; //$NON-NLS-1$

				this._resizeHandle = jazz.util.ResizeNotifier.register({
					context: this,
					callback: this._resizeSoon,
					force: this._force,
					priority: this._priority});
				
				if (args.listenOnPage) {
					this._listenOnPage = args.listenOnPage;
					dojo.subscribe(net.jazz.ajax.internal.events.viewPage, this, this._pageDidChange);
				}

				if (args.scrollDownClass) {
					this._onScrollHandle = dojo.connect(box, 'onscroll', this, this._viewDidScroll); //$NON-NLS-1$
					this._scrollDownClass = args.scrollDownClass;
				}
				window.setTimeout(dojo.hitch(this, this.resize), 1);
			},

			/**
			 * When enabled, will track the mouse position when it moves. If it's near
			 * the top or bottom of the scrolling box, then the box will automatically scroll
			 * in that direction. This is used with drag-and-drop.
			 */
			enableAutoScroll: function() {
				var pos = dojo.position(this.box);
				
				// Assume the page itself doesn't scroll or resize while tracking
				// auto scroll. Just cache the absolute page positions for fast region hit
				// checking later.				
				this._topRegion = dojo.mixin({}, pos, {
					h: 40,
					y: pos.y - 20
				});

				this._bottomRegion = dojo.mixin({}, pos, {
					h: 40,
					y: pos.y + pos.h - 20
				});

				this._autoScrollHandle = dojo.connect(document, 'onmousemove', this, this._onAutoScroll); //$NON-NLS-1$
			},

			/**
			 * Disables autoscrolling. Only needs sometime after a call to .enableAutoScroll.
			 */
			disableAutoScroll: function() {
				dojo.disconnect(this._autoScrollHandle);
				delete this._autoScrollHandle;
				clearInterval(this._scrollUpInterval);
				clearInterval(this._scrollDownInterval);
				delete this._bottomRegion;
				delete this._topRegion;
			},

			/**
			 * On mousemove, this detects if it's in an auto scroll region
			 * and scrolls if so.
			 * @param  {mouse event} e
			 */
			_onAutoScroll: function(e) {
				var box = this.box;

				if (util.isEventWithin(e, this._bottomRegion)) {
					clearInterval(this._scrollUpInterval);

					if (!this._scrollDownInterval) {
						this._scrollDownInterval = setInterval(function() {
							box.scrollTop += 10;
						}, 	20);
					}
				}
				else if (util.isEventWithin(e, this._topRegion)) {
					clearInterval(this._scrollDownInterval);

					if (!this._scrollUpInterval) {
						this._scrollUpInterval = setInterval(function() {
							box.scrollTop += -10;
						}, 	20);
					}
				}
				else {
					clearInterval(this._scrollDownInterval);
					delete this._scrollDownInterval;

					clearInterval(this._scrollUpInterval);
					delete this._scrollUpInterval;
				}
			},


			/**
			 * Callback function after a resize that can be listened to.
			 * @param  {object} newSize object with .w and .h properties for
			 *     the new size.
			 */
			didResize: function(newSize) {
				// callback function
			},

			_viewDidScroll: function(event) {
				if (this.box.scrollTop !== 0) {
					dojo.addClass(this.box, this._scrollDownClass);
				}
				else {
					dojo.removeClass(this.box, this._scrollDownClass);
				}
			},

			_pageDidChange: function(page) {
				if (!this._resizeHandle && this._listenOnPage == page.id) {
					this._resizeHandle = jazz.util.ResizeNotifier.register({
						context: this,
						callback: this._resizeSoon,
						force: this._force,
						priority: this._priority
					});

					// Resize in case the window changed size while we were out of focus.
					this._resizeSoon();
				}
				else if (this._resizeHandle && this._listenOnPage != page.id) {
					jazz.util.ResizeNotifier.unregister(this._resizeHandle);
					delete this._resizeHandle;
				}
			},

			/**
			 * Sets a short timeout until the resize is called. This seems to give more
			 * accurate resizes (I think the Jazz notifier misses the last event sometimes).
			 * It also results in fewer resizes, which may make window resizeing smoother
			 * on slower browsers.
			 */
			_resizeSoon: function() {
				// If there is already a pending resize, ignore this event to
				// limit the number or resizes we have to do.
				if (this._resizeTimeout) return;
				this._resizeTimeout = window.setTimeout(dojo.hitch(this, this.resize), 100);	
			},

			/**
			 * Automatically called after a resize event, but can be called at anytime
			 * to force a resize now.
			 */
			resize: function() {
				window.clearTimeout(this._resizeTimeout);
				// Delete this since it's used as a flag to know if a resize is pending or not.
				delete this._resizeTimeout;

				try {
					var boxPos = dojo.position(this.box);
					var contentPos = dojo.position('content-cell'); //$NON-NLS-1$
					var yOffset = boxPos.y - contentPos.y;

					var newSize = { h: contentPos.h - yOffset };

					if (this._fillParentWidth) {
						// This basically assumes we're in the
						var parentWidth = dojo.contentBox(this.box.parentNode).w;
						newSize.w = parentWidth;
					}

					dojo.marginBox(this.box, newSize);
					this.didResize(newSize);
				}
				catch (e) {
					// IE will throw if you measure the size of something that's hidden.
					// Ideally we should not even try this for hidden boxes (and we try to do that with
					// page change events above) but it doesn't seem reliable. Maybe we should revist this
					// but just hiding the error is OK for now.
				}
			},

			/**
			 * Must be called when done with this resizer to unregister listening for events.
			 */
			destroy: function() {
				window.clearTimeout(this._resizeTimeout);
				this._resizeHandle && jazz.util.ResizeNotifier.unregister(this._resizeHandle);
				dojo.disconnect(this._hoverHelp);
			}
		}	
	);

})();