dojo.provide('com.ibm.relm.common.web.ui.LqeAuthInterceptor'); //$NON-NLS-1$

/**
 * XHR interceptor that will automatically open the LQE user authorization page
 * in response to an error header from LQE. Requiring this module will automatically
 * register it as a response interceptor for normal jazz.client.xhr* methods.
 * 
 * XHR requests will automatically be retried after the auth window is closed by the user
 * or they have waited too long to log in, normally making the authorization
 * transparent to the called.
 */

dojo.require('jazz.client.xhr'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ui.LqeAuthDialog'); //$NON-NLS-1$

(function() {

/**
 * Queue of all requests that failed with a need for LQE authorization. Once
 * the auth dialog is closed, these will all be retried and the queue will be
 * cleared.
 */
var requestQueue_ = [];

/**
 * Handle to the currently open LqeAuthDialog.
 * Used only as a way to know the dialog is already open
 * and shouldn't be reopened.
 */
var openAuthDialog_ = null;

/**
 * Callback after dialog closes. Retries all pending requests.
 */
function onAuthDialogClose() {
	openAuthDialog_ = null;

	while (requestQueue_.length) {
		// Prevent endless loop of retries. If we failed once, the next time
		// will/should just propogate the error instead of running this interceptor again.
		var origRequest = requestQueue_.pop().request;
		origRequest.args.com_ibm_relm_lqeAuthSecondTry = true;
		origRequest.args.load = function (result) {
			origRequest.clientDeferred.callback(result);
		};
		origRequest.args.error = function (error, ioArgs) {
			origRequest.clientDeferred.errback(error, ioArgs);
		};
		jazz.client.xhr(origRequest.method, origRequest.args, origRequest.hasBody);				
	}
}

/**
 * [lqeAuthInterceptor description]
 * @param  {object} originalResponse
 *         object with original .xhrArgs and error .response and .ioArgs to
 *         the request can be retried. Passed straight through from
 *         jazz.client RequestManager.
 * @param {object} origResponse
 *        the failed response. Can be used to invoke the errback and send back
 *        the response.
 * @param  {[type]} requestManager [description]
 */
function lqeAuthInterceptor(origRequest, origResponse, requestManager) {
	// Skip this on the second try. We don't want an endless loop of LQE auth dialogs
	// popping up. We either already authorized with LQE this this will work without
	// intervention, or we just let it fail this time.
	if (origRequest.args.com_ibm_relm_lqeAuthSecondTry) return false;

	var lqeAuthUrl = origResponse.getHeader('X-lqe-web-oauth-url'); //$NON-NLS-1$

	if (!lqeAuthUrl) {
		return false;
	}

	// Queue all requests. Important to maintain them all for views which
	// can do several parallel queries.
	requestQueue_.push({
		request: origRequest,
		response: origResponse
	});

	// We're waiting for authorization already. Just queue this, but
	// don't open a new auth window.
	if (openAuthDialog_) {
		return true;
	}

	openAuthDialog_ = new com.ibm.relm.common.web.ui.LqeAuthDialog({
		lqeAuthUrl: lqeAuthUrl,
		onClose: onAuthDialogClose
	});

	return true;
}

jazz.client.internal.addResponseInterceptor(lqeAuthInterceptor);

})();