/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2010. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.common.web.ui.Section"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require("dojo.fx"); //$NON-NLS-1$
dojo.require("jazz.app.ua.PageHoverHelp"); //$NON-NLS-1$

/**
 * A "section" of a larger page. Visually, has a horizontal header bar above a with a title for
 * the section. This mirrors the style the Work Items UI. The actual content below the title
 * is not visually contained in a border: the header and trailing white space should be used
 * to indicate the containment of the content.
 *
 * Optionally, the section supports being collapsed when clicked. This is rarely useful behavior though,
 * (it is generally faster to scroll than collapse, especially if sections don't remember their collapsed
 * states).
 *
 * Section can be used directly in an HTML template. The nested content will be seen in the
 * main, collapsible section.
 */
(function(){
	dojo.declare(
		"com.ibm.relm.common.web.ui.Section", //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ], 
		{
			templatePath: dojo.moduleUrl('com.ibm.relm.common.web','ui/templates/Section.html'), //$NON-NLS-1$ //$NON-NLS-2$
			widgetsInTemplate: true,

			label: '', //$NON-NLS-1$
			canCollapse: false,
			initiallyCollapsed: false,
			iconClass: '', //$NON-NLS-1$
			inSidebar: false,
			htmlFor: '', //$NON-NLS-1$
			//By default the section will show the help. If the help should be hidden, then set this to false in constructor.
			showHelp: true,
			uaKey:'', //$NON-NLS-1$

			/**
			 * Params can be set in the template.
			 * @param label the label shown in the header of the section.
			 * @param canCollapse set to true to allow the section to be collapsed when the header is clicked.
			 */
			constructor: function(args, node) 
			{
				this._collapsed = false;
				this._inSidebar = args.inSidebar;
				this._helpArgs = args.helpArgs;
			},

			onCollapse: function() {},
			
			postCreate: function() {
				this.inherited(arguments);

				if (this._inSidebar) {
					dojo.addClass(this.domNode, 'relm-Section-in-sidebar'); //$NON-NLS-1$
				}
				
				if (this.canCollapse) {
					dojo.addClass(this.domNode, 'relm-Section-can-collapse'); //$NON-NLS-1$
					this._titleNode.tabIndex = 0;

					if (this.initiallyCollapsed) {
						this._collapsed = true;
						dojo.addClass(this.domNode, 'relm-Section-collapsed'); //$NON-NLS-1$
						this._containerWrapper.style.display = 'none'; //$NON-NLS-1$
					}
				}

				if (this.showHelp) {
					if (!this.uaKey)
						console.warn("No uaKey provided to hover help"); //$NON-NLS-1$
					this._hoverHelp = new jazz.app.ua.PageHoverHelp({uaKey: this.uaKey},this._helpHolder);
					this.connect(this._hoverHelp.domNode, 'onclick', this.onHelpClick); //$NON-NLS-1$
				}

				if (this.htmlFor) {
					dojo.attr(this._titleNode, 'for', this.htmlFor); //$NON-NLS-1$
				}
			},

			isCollapsed: function() {
				return this._collapsed;
			},

			onHelpClick: function(e) {
				dojo.stopEvent(e);
			},
			
			setLabel: function(label) {
				this._titleNode.innerHTML = label; 
			},
			
			/**
			 * Adds an DOM node to the right side of the header for control that apply to the entire section.
			 * Note that toolbar buttons are too tall for the header normally. In other Jazz apps, hyperlinks
			 * are normally used as the controls in this section.
			 *
			 * If the control is a link and is given a class of "relm-Section-link" then some common styling will be applied.
			 */
			setControls: function(node) {
				dojo.empty(this._controlsNode);
				this._controlsNode.appendChild(node);
			},

			addControls: function(node) {
				this._controlsNode.appendChild(node);
			},
			/**
			 * Programatically collapses or uncollapses the section.
			 */
			collapse: function(e) {
				if (!this.canCollapse) return;
				if (e && e.keyCode) {
					// If there is a keypress, then do collapse of spacebar or enter. If no key, then it's likely a mouse=
					// click and still do it.
					if (e.keyCode !== dojo.keys.ENTER && e.keyCode !== dojo.keys.SPACE) return;
				}

				var wipeArgs = {
					node: this._containerWrapper,
					duration: 200,
					onEnd: dojo.hitch(this, this.onCollapse, !this._collapsed)
				};

				if (this._collapsed) {
					dojo.fx.wipeIn(wipeArgs).play();
					this._collapsed = false;
					dojo.removeClass(this.domNode, 'relm-Section-collapsed'); //$NON-NLS-1$

				}
				else {
					dojo.fx.wipeOut(wipeArgs).play();
					this._collapsed = true;
					dojo.addClass(this.domNode, 'relm-Section-collapsed'); //$NON-NLS-1$
				}
			},

			uninitialize: function() {
				this._hoverHelp && this._hoverHelp.destroyRecursive();
			}
		}	
	);

})();