/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.Site'); //$NON-NLS-1$

(function(){

	function itemPathToPath(itemPath) {
		return dojo.map(itemPath, function(item) {
			return item.pathSegment;
		});
	}

	function findContextConfigVersion(itemPath) {
		// Walk backwards up the path until we find a context config version.
		for (var i = itemPath.length - 2; i >= 0; i--) {
			var tail = itemPath[i];

			if (tail.type === 'configVersion') { //$NON-NLS-1$
				return itemPath[i];
			}
		}

		return null;
	}

	/**
	 * A site is just an object with properties describing its location in the
	 * tree and details about the selection. Sites have the following
	 * properties (updated significantly for 5.0): contextConfigVersionUri: If
	 * the selection is in the context of a configuration, this is the URI of
	 * that configuration. Root configurations have no context configuration
	 * (they are not in one), but all item versions and child configurations
	 * do.
	 *
	 * A site is created from an item path, that is, an array of JSON items
	 * from a root configuration to the descendant selected item.
	 *
	 * contextConfigVersion: The JSON item for the context config version.
	 * This is used to show the context configuration when, say, creating a
	 * new product or showing the product details view where the context
	 * configuration is important for the user to see.
	 * 
	 * contextPartUri: If the selection has a `contextConfigVersionUri`, then
	 * it has a contextPartUri as well. Both are needed to identify the exact
	 * version of a child within a configuration. Many API method take a Java
	 * class call OptionalPartContext (sometimes PartContext) which wraps
	 * these two URIs together. (Note that the Java API for .getTree overloads
	 * the use of the "context configuration" and will treat it as a root
	 * configuration if no part URI is provided--this JavaScript class is more
	 * strict about always using the term "context" for configurations above
	 * the current selection.)
	 *
	 * link: If the selected node is a link to an artifact, this will be a
	 * JSON item describing the link. Links are different than other nodes
	 * since they are nodes in the tree, but are just implemented as
	 * properties of an item version or config version. Thus, if a link is
	 * selected, either the `itemVersion` or `configVersion` property will
	 * also be set to describe the actual object being modified.
	 * 
	 * configVersion: If the selected node is a configuration version (or a
	 * link directly under the config version), this will be a JSON item for
	 * the selected config version. This includes root config version nodes
	 * and child config version nodes. When a root config version is selected,
	 * there is no `contextConfigVersionUri` or `contextPartUri`. Mutually
	 * exclusive with `itemVersion`.
	 *
	 * configVersionUri: Just a convenience to skip null checks in some cases.
	 * If `configVersion` is set, this is set to its URI.
	 *
	 * itemVersion: If the selected node is an item version (or a link
	 * directly under the item version), this will be a JSON item for the
	 * selected item version. Mutually exclusive with `configVersion`. Item
	 * versions are always under a configuration in the tree, so
	 * `contextConfigVersionUri` is guaranteed to be set. Note that 4.0.6 use
	 * the property `product` for this case. `itemVersion`is now the preferred
	 * property name.
	 * 
	 * itemVersion: Just a convenience to skip null checks in some cases. If
	 * `itemVersion` is set, this is set to its URI.
	 *
	 * emptyPart: If the selected node is an empty part, that is, VVC does not
	 * selected a child item version or config version, then this is set to an
	 * item describing the empty part. Empty parts do have names, but no URL:
	 * interact with them via the context part and context config version.
	 * 
	 * parentResourceVersion: If the selected node has a parent in the tree,
	 * this is set to that parent JSON item, be it a config version or item
	 * version. Is null for top-level config versions. If a link is selected,
	 * this is the parent of the item version of config version that owns the
	 * link--not the link owner itself. That is consistent with the view the
	 * links are not resources: they are properties of the item version or
	 * config version visually above them in the tree. (This property fills
	 * the same role as `parentProduct` did in 4.0.6, but can now potentially
	 * be a config version. It is generally needed to know this for move and
	 * reuse operations, but hasn't be used yet in 5.0, so is subject to
	 * change).
	 *
	 * path: The currently selected path in the tree as an array of
	 * `pathSegment` properties. Used mostly for changing the currently
	 * selected path by setting the tree's current path to an altered version
	 * of this path (e.g., adding a new `pathSegement` to selected a newly
	 * added child). Note the tree can have multiple nodes selected, in which
	 * case there are multiple site objects created, each with their own path
	 * property.
	 *
	 * itemPath: The currently selected path in the tree as an array or JSON
	 * items. Used if more details information is needed about some object in
	 * the tree up the ancestor chain (e.g. am I under an RTC connected
	 * node?).
	 *
	 * isInBaseline: true only if its context config version is a baseline. Baselines
	 * are not auto-checked out, so all mutating operations should be disabled
	 * in them.
	 * 
	 * product: Legacy name for `itemVersion`. Don't use in new code. Will
	 * likely live on in 5.0.
	 * 
	 * parentProduct: Legacy name for `parentResourceVersion`. Doesn't
	 * accurately reflect parent config versions. Code still using this is
	 * likely to be broken in 5.0. To be removed.
	 */
	com.ibm.relm.ple.web.ui.Site = function(itemPath) {
		if (itemPath.length === 0) throw new Error('Invalid site, no item path given.'); //$NON-NLS-1$

		var contextConfigVersionUri;
		var contextPartUri;
		var configVersion;
		var configVersionUri;
		var itemVersion;
		var itemVersionUri;
		var link;
		var emptyPart;
		var parentResourceVersion;
		var isInBaseline;

		var selectedItem = itemPath[itemPath.length - 1];
		var selectedType = selectedItem.type;

		// Type is special. In the tree it looks like a node, but it's really
		// a property of its parent in the API. So, if a link is selected,
		// effectively, it's parent is really selected, but with the extra context
		// of the selected link.
		if (selectedType === 'link') { //$NON-NLS-1$
			link = selectedItem;
			var linkParent = itemPath[itemPath.length - 2];
			var effectivelySelectedItem = linkParent;
			parentResourceVersion = itemPath[itemPath.length - 3];
		}
		else {
			effectivelySelectedItem = selectedItem;
			parentResourceVersion = itemPath[itemPath.length - 2];
		}

		
		// Item versions and child config versions will already have these set on them.
		contextConfigVersionUri = effectivelySelectedItem.contextConfigVersionUri;
		contextPartUri = effectivelySelectedItem.contextPartUri;

		// Is in Baseline property is set based only on context configuration, not selected configuration
		var contextConfigVersion = findContextConfigVersion(itemPath);
		if (contextConfigVersion) isInBaseline = contextConfigVersion.isImmutable;

		var effectivelySelectedType = effectivelySelectedItem.type;

		if (effectivelySelectedType === 'configVersion') { //$NON-NLS-1$
			configVersion = effectivelySelectedItem;
			configVersionUri = configVersion.uri;
		}
		else if (effectivelySelectedType === 'product') { //$NON-NLS-1$
			itemVersion = effectivelySelectedItem;
			itemVersionUri = itemVersion.uri;

			// Legacy name is 'product'
			var product = itemVersion;
		}
		else if (effectivelySelectedType === 'emptyPart') { //$NON-NLS-1$
			emptyPart = effectivelySelectedItem;
		}
		else {
			throw new Error('Unknown selected type: ' + effectivelySelectedType); //$NON-NLS-1$
		}

		// XXX: The use of 'parentProduct' is questionable in 5.0. I think this will always need to be "context part" or //$NON-NLS-1$ //$NON-NLS-2$
		// similar at some point.
		var parentProduct = parentResourceVersion;

		// Now set all local properties
		this.type = selectedType;
		this.resourceVersionUri = configVersionUri || itemVersionUri;

		this.contextConfigVersionUri = contextConfigVersionUri;
		this.contextPartUri = contextPartUri;

		this.configVersion = configVersion;
		this.configVersionUri = configVersionUri;

		this.itemVersion = itemVersion;
		this.itemVersionUri = itemVersionUri;

		this.link = link;

		this.emptyPart = emptyPart;

		// the parent above either configVersion or itemVersion (whichever is set).
		// Can be null if configVersion is selected and it's a top-level config version.
		this.parentResourceVersion = parentResourceVersion;
		this.isInBaseline = isInBaseline;

		this.path = itemPathToPath(itemPath);
		this.itemPath = itemPath;

		// legacy names from 4.0.6. Will be removed
		this.product = itemVersion;
		this.parentProduct = parentProduct;
	};


	/**
	 * A utility method to help update the current ID path (used in the address bar)
	 * after a check out or other operation that may modify an item version.
	 * This to be used as a method on a `site` object (it is added in .newSitesFromItemPaths).
	 *
	 * When called on a site, it will create a path to an item version
	 * at the same tree depth. That is, it will take the parent ID path
	 * of the site, and add the pathSegment of a new item version. This is
	 * used when refreshing after some modify operation. In many cases the
	 * resulting path will be the same since "part equivalence IDs"
	 * should not change during a check out. This is used for
	 * historical reasons and since it's not clear if there are still
	 * cases where it is truly needed.
	 * 	
	 * @param  {product} otherProduct the other product to get the pathSegment from to construct
	 *                                	the full new path.
	 * @return {array}              an ID path similar to the one on the starting site, but
	 *                                 with a potentially different leaf if otherProduct
	 *                                 had a different pathSegment.
	 */
	 com.ibm.relm.ple.web.ui.Site.prototype.pathForPeer = function(peerResource) {
	 	var otherPath = this.path.slice(0, -1);
	 	otherPath.push(peerResource.pathSegment);
	 	return otherPath;
	 };

	com.ibm.relm.ple.web.ui.Site.prototype.closestConfigVersion = function() {
		if (this.configVersion) return this.configVersion;

		return findContextConfigVersion(this.itemPath);
	};

	/**
	 * Converts a Site into a simpler object that can be reliably serialized
	 * to JSON and sent to the Java server. Only a few critical URIs are kept.
	 * @param  {Site} site to convert.
	 * @return {object}      simpler site style object with only critical URIs.
	 */
	function _siteToServerSafeSite(site) {
		var safeSite = {};

		if (site.contextConfigVersionUri) safeSite.contextConfigVersionUri = site.contextConfigVersionUri;
		if (site.contextPartUri) safeSite.contextPartUri = site.contextPartUri;
		if (site.configVersionUri) safeSite.configVersionUri = site.configVersionUri;
		if (site.itemVersionUri) safeSite.itemVersionUri = site.itemVersionUri;
		if (site.link) safeSite.linkUri = site.link.uri;

		return safeSite;
	}

	/**
	 * Converts a site into a JSON string safe for sending to the server.
	 * @param  {Site} site to serialize.
	 * @return {string}      JSON site.
	 */
	com.ibm.relm.ple.web.ui.Site.siteToJsonSite = function(site) {
		if (!site) return null;
		var safeSite = _siteToServerSafeSite(site);
		return dojo.toJson(safeSite);
	};
	
	/**
	 * Converts several sites into a JSON array string safe for sending to the
	 * server.
	 * @param  {array} sites array of Sites.
	 * @return {string}       JSON array of sites.
	 */
	com.ibm.relm.ple.web.ui.Site.sitesToJsonSites = function(sites) {
		var safeSites = [];

		for (var i = 0; i < sites.length; i++) {
			var site = sites[i];
			safeSites.push(_siteToServerSafeSite(site));
		}

		return dojo.toJson(safeSites);
	};

	/**
	 * Converts and item path to a simple path, that is, from an array
	 * of item objects, to an array of their path segmetns.
	 * @param {array} item path.
	 * @return {array} a path. Array of the path segments of the items.
	 */
	com.ibm.relm.ple.web.ui.Site.itemPathToPath = itemPathToPath;

})();