dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.pages', 'ProductsPageMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.provide('com.ibm.relm.ple.web.ui.TreeSelection'); //$NON-NLS-1$

dojo.require('com.ibm.relm.ple.web.ui.Site'); //$NON-NLS-1$

(function(){

var Site = com.ibm.relm.ple.web.ui.Site;
var messages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.pages', 'ProductsPageMessages'); //$NON-NLS-1$ //$NON-NLS-2$
var selectedSites_ = [];



/**
 * This is a global location to track the currently selected item or items in
 * the product tree. This class uses the term "site" or "tree site" to refer
 * to the objects used to store selections. A site is just an object with
 * properties describing its location in the tree and details about the
 * selection. See the Site class for details.
 */
dojo.mixin(com.ibm.relm.ple.web.ui.TreeSelection, {

	/**
	 * The data store that that the tree (or future widget) is using
	 * to access items in it. Any called that reads the selected items
	 * from this object must access their properties via the store methods.
	 * 
	 * This feels pretty fragile, since the tree must set this once it's loaded.
	 * Once set, this never changes.
	 * 
	 * @type {data.store}
	 */
	store: null,

	setSelectedSitesFromItemPaths: function(itemPaths) {
		var newSelection = this.newSitesFromItemPaths(itemPaths);
		this.setSelectedSites(newSelection);
	},

	/**
	 * Creates an array of "site" object structures. This doesn't
	 * actually set them as the global selection, it just creates the objects.
	 * This is used internally when setting them, but can also be called
	 * for creating artificial "selections" for drag and drop.
	 * @param  {array} itemPaths array of JSON items in path order from root (config version) to selected items.
	 * @return {object}           site description object including selected resource, link and config version details.
	 */
	newSitesFromItemPaths: function(itemPaths) {
		var selectedSites = [];

		dojo.forEach(itemPaths, function(itemPath) {
			selectedSites.push(new Site(itemPath));
		}, this);

		return selectedSites;
	},

	/**
	 * Takes a new array of selected objects and sets them as the current
	 * selection. Notifies any listeners.
	 * @param {array} newSelection array of objects with .parentProduct and .product fields on them.
	 * Array of 1 if there is only one selected item, array of more if multi-select was used.
	 * Can be emtpy array for no selection.
	 */
	setSelectedSites: function(newSelection) {
		selectedSites_ = newSelection.slice(0);
		this.onSelectionChange();
	},

	/**
	 * Clears all the selected paths. The selection will be an empty array.
	 */
	clearSelectedSites: function() {
		this.setSelectedSites([]);
	},

	/**
	 * Gets all the selected items, assuming multi-select was used
	 * and there is more than one. See .getSelected for details.
	 * @return {array}          array of 0, 1, or more selected objects.
	 */
	getAllSelectedSites: function() {
		return selectedSites_.slice(0);
	},

	/**
	 * Gets the main selected object. If multi select was used, this is the last selected object,
	 * and the current focused object.
	 * 
	 * @return {object} the selected site, or null if there is no selection. The last
	 * selected site if multiple sites are selected. See class description for site details.
	 */
	getSelectedSite: function() {
		if (selectedSites_.length > 0) {
			return selectedSites_[selectedSites_.length - 1];
		}
		else {
			return null;
		}
	},

	/**
	 * Like .getSelectedSite, but first asynchronously loads a fresh copy of the selected item
	 * version or config version from the server before returning the site.
	 * Will always include the user properties on the loaded item (a normal
	 * site from the user tree does not have that information).
	 * This is used when bringing up Edit Properites and similar dialogs
	 * to ensure the user is seeing and editing an up to date copy
	 * of the resource.
	 * 
	 * @return {dojo.Deferred} deferred that calls back once
	 * the loaded versioned resource is available. Errrback with
	 * an error object if the load fails.
	 */
	loadSelectedSiteWithUserProperties: function() {
		var deferred = new dojo.Deferred();

		var site = this.getSelectedSite();
		if (!site) {
			deferred.callback(null);
			return deferred;
		}

		var contextConfigVersionUri;
		var contextPartUri;

		if (site.configVersion) {
			contextConfigVersionUri = site.configVersionUri;
		}
		else {
			contextConfigVersionUri = site.contextConfigVersionUri;
			contextPartUri = site.contextPartUri;
		}

		var message = com.ibm.relm.common.web.StatusMessage.show(messages.loadingNoEllipsis);

		// Must get the latest version on Edit
		jazz.client.xhrGet({
			handleAs: 'json', //$NON-NLS-1$
			url: net.jazz.ajax._contextRoot + '/ple/get_resource_version', //$NON-NLS-1$
			content: {
				contextConfigVersionUri: contextConfigVersionUri,
				contextPartUri: contextPartUri,
				includeUserProperties: true
			},

			handle: function() {
				message.hide();
			},

			load: function (resource) {
				// take the fresh copy of the resource and stick it back into
				// the site struct. This could be either a config version or item version.
				resource.contextConfigVersionUri = contextConfigVersionUri;
				resource.contextPartUri = contextPartUri;

				if (site.configVersion) {
					site.configVersion = resource;
				}
				else {
					site.itemVersion = resource;
				}

				deferred.callback(site);
			},

			error: function(error, ioArgs) {
				var errorObject = dojo.fromJson(ioArgs.xhr.responseText);
				deferred.errback(errorObject);
			}
		});

		return deferred;
	},

	/**
	 * Convenience function to return the 'type' of the currently selected item,
	 * that is, the type of the .product or .link (if a link is selected) that would be returned by .getSelected.
	 * @return {string} null if no selection, otherwise the type of the selected leaf.
	 */
	getSelectedType: function() {
		if (selectedSites_.length === 0 || !this.store) {
			return null;
		}

		var leaf = selectedSites_[selectedSites_.length - 1];
		return (leaf.link || leaf.itemVersion || leaf.configVersion || leaf.emptyPart).type;
	},

	/**
	 * For all selected nodes, check that they are all product types (no links.)
	 * @return {boolean} true if all selected nodes are products.
	 */
	areAllSelectedProducts: function() {
		return dojo.every(selectedSites_, function(selected) {
			return !selected.link;
		}, this);
	},

	/**
	 * Callback functiom. Use dojo.connect to listen to changes to the
	 * selection. No parameters are passed to the callback, so interested parties
	 * must use other TreeSelection methods to inpsect the selection.
	 */
	onSelectionChange: function() {
		// no-op callback
	}

});	

})();