/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.ple.web.ui.history.HistoryGrid"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.ple.web.ui.history", "HistoryGridMessages"); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dojo.NodeList-traverse'); //$NON-NLS-1$
dojo.require('dojox.gfx'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.formatting'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.audit.ProductAudit'); //$NON-NLS-1$
dojo.require('jazz.ui.toolbar.Button'); //$NON-NLS-1$
dojo.require('jazz.ui.internal.ResourceLinkAggregator'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.ResourceLinkBatchLoader'); //$NON-NLS-1$
dojo.require('relm.app.Permissions'); //$NON-NLS-1$
dojo.require('com.ibm.relm.ple.web.ui.productapi'); //$NON-NLS-1$

(function(){
	var Permissions = relm.app.Permissions;

	function isArrowKey(keyCode) {
		var k = dojo.keys;
		return keyCode === k.LEFT_ARROW || keyCode === k.RIGHT_ARROW || keyCode === k.UP_ARROW || keyCode === k.DOWN_ARROW;
	}

	function closest(domNode, tagName) {
		while (domNode && domNode != window) {
			if (domNode.nodeName === tagName) return domNode;
			domNode = domNode.parentNode;
		}

		return null;
	}

	/**
	 * Checks if a mouse event looks like it's a right-click. Right-click will
	 * send a contextmenu event, but can also send a mousedown event that we
	 * may want to ignore when there is a dedicated contextmenu event handler.
	 * 
	 * @param  {mouse event}  event
	 * @return {Boolean}      true if it's a right-click event.
	 */
	function isRightClick(event) {
		if (event.which) {
			return event.which === 3;
		}
		else if (event.button) {
			return event.button === 2;
		}

		return false;
	}

	/**
	 * Finds to the top-center point of a box.
	 * @param  {object} position result of a dojo.position.
	 * @return {object}          point with x and y fields.
	 */
	function topCenter (position) {
		return {
			x: Math.ceil(position.x + position.w / 2),
			y: position.y
		};
	}

	/**
	 * Finds to the bottom-center point of a box.
	 * @param  {object} position result of a dojo.position.
	 * @return {object}          point with x and y fields.
	 */
	function bottomCenter(position) {
		return {
			x: Math.ceil(position.x + position.w / 2),
			y: position.y + position.h
		};
	}

	/**
	 * Given two point, returns their vector, that is,
	 * a single point from 0, 0 with the same length and angle.
	 * @param  {object} startPosition start point.
	 * @param  {object} endPosition   end point.
	 * @return {object}               end point of the vector.
	 */
	function vector(startPosition, endPosition) {
		return {
			x: endPosition.x - startPosition.x,
			y: endPosition.y - startPosition.y
		};
	}

	/**
	 * Given two points, finds the angle of the line between them.
	 * @param  {object} startPosition start point.
	 * @param  {object} endPosition   end point.
	 * @return {number}               angle of the line in radians from positive x axis.
	 */
	function angle(startPosition, endPosition) {
		var v = vector(startPosition, endPosition);
		return Math.atan2(v.y, v.x);
	}

	/**
	 * IE 8 can take a long time to draw the arrows.
	 * We'll keep checking the time while it's drawing. Once we cross
	 * this threshold, we'll just stop drawing arrows. Not perfect,
	 * but scales. Will attempt to still draw arrows on demand as nodes are clicked.
	 * @type {Number}
	 */
	var MAX_DRAW_TIME_MILLIS = 7000;

	/**
	 * The fixed width of branch columns. This makes layout faster and easier,
	 * since we don't need to redraw arrows when the window resizes. Much faster in IE.
	 * It also gives a uniform appearance, but doesn't optimize for the type of content.
	 * @type {Number}
	 */
	var BRANCH_COLUMN_CSS_PX = 400;

	/**
	 * The main part of the history view. The grid (table really) of all products.
	 * Creates the table to hold them, the boxes for product, and the arrows
	 * to show successor relationships.
	 * 
	 * Handles events for clicking and selecting products.
	 * 
	 * Several methods use "point" objects, which are just object
	 * with a x and y property, e.g., {x: 1, y: 5}. Position objects
	 * should match those returned from dojo.position, which have x, y, h, and w.
	 */
	dojo.declare(
		"com.ibm.relm.ple.web.ui.history.HistoryGrid", //$NON-NLS-1$
		[ dijit._Widget ],
		{
			baseClass: 'ple-HistoryGrid', //$NON-NLS-1$
			messages: dojo.i18n.getLocalization("com.ibm.relm.ple.web.ui.history", "HistoryGridMessages"), //$NON-NLS-1$ //$NON-NLS-2$

			constructor: function(args, node) {
				this._selectedProducts = [];
				this._urisToProducts = {};
				this._audits = [];
				this._buttons = [];
				this._productNodeHandles = [];
				this._batchLoader = new com.ibm.relm.common.web.ResourceLinkBatchLoader();
			},

			_destroyAudits: function() {
				dojo.forEach(this._audits, function(audit) {
					audit.destroyRecursive();
				});

				this._audits = [];
			},

			_destroyButtons: function() {
				dojo.forEach(this._buttons, function(b) {
					b.destroyRecursive();
				});

				this._buttons = [];
			},

			postCreate: function() {
				this.domNode.innerHTML = '&nbsp;'; //$NON-NLS-1$

				this._table = dojo.create('table', { //$NON-NLS-1$
					unselectable: "on", //$NON-NLS-1$
					className: 'ple-HistoryGrid-grid', //$NON-NLS-1$
					role: 'presentation' //$NON-NLS-1$
				}, this.domNode);

				this.connect(this.domNode, 'onclick', this.onClickProduct); //$NON-NLS-1$
				this.connect(this.domNode, 'oncontextmenu', this.onClickProduct); //$NON-NLS-1$

				// Heavy handed text selection prevention. IE otherwise will select text during pans.
				this.connect(this.domNode, 'onselectstart', dojo.stopEvent); //$NON-NLS-1$
				this.connect(this.domNode, 'ondragstart', dojo.stopEvent); //$NON-NLS-1$

				this.connect(this.domNode, 'onkeydown', this._handleKeyDown); //$NON-NLS-1$
			},

			_handleKeyDown: function(e) {
				var c = e.keyCode;
				var k = dojo.keys;

				// Toggle selection of focus node
				switch (c) {
				case k.SPACE:
				case k.ENTER:
					if (e.target.nodeName === 'A') return; //$NON-NLS-1$

					if (this._lastFocusedProductNode) {
						dojo.stopEvent(e);
						var productUri = this._lastFocusedProductNode['data-productUri']; //$NON-NLS-1$
						var product = this._urisToProducts[productUri];
						this._selectProduct(product, c === k.SPACE || e.ctrlKey);
						return;
					}
					break;
				default:
					break;
				}

				if (isArrowKey(c)) {
					var coords = this._getLastFocusedProductNodeCoords();
					if (coords) {
						dojo.stopEvent(e);

						switch(c) {
						case k.LEFT_ARROW:
							var nextNode = this._nextHorizonalProductNodeInTable(coords.row, coords.cell, true);
							break;
						case k.RIGHT_ARROW:
							nextNode = this._nextHorizonalProductNodeInTable(coords.row, coords.cell, false);
							break;
						case k.UP_ARROW:
							nextNode = this._nextVerticalProductNodeInTable(coords.row, coords.cell, true);
							break;
						case k.DOWN_ARROW:
							nextNode = this._nextVerticalProductNodeInTable(coords.row, coords.cell, false);
							break;
						default:
							return;
						}

						nextNode && nextNode.focus();
					}
				}
			},

			/**
			 * Finds the next occupied product node in the table left or right
			 * of a given location.
			 * @param  {integer} fromRow
			 * @param  {integer} fromCol
			 * @param  {boolean} reverseDirection if true, search backwards (left).
			 * @return {node}                  product node or null if not found.
			 */
			_nextHorizonalProductNodeInTable: function(fromRow, fromCol, reverseDirection) {
				if (!this._table || !this._table.firstChild) return null;

				var tbody = this._table.firstChild;
				var rows = tbody.rows;

				var increment = reverseDirection ? -1 : 1;
				var cells = rows[fromRow].cells;
				var cellIndex = fromCol + increment;

				for (; cellIndex < cells.length && cellIndex >= 0; cellIndex += increment) {
					var cell = cells[cellIndex];
					var productNode = dojo.query('.ple-HistoryGrid-product', cell)[0]; //$NON-NLS-1$
					if (productNode) return productNode;
				}

				return null;
			},


			/**
			 * Finds the next occupied product node in the table above or
			 * below a given location. The next row and same column are
			 * checked first. If there is no product in the column, the same
			 * row is checked first to the left then the right for the first
			 * product so that some product on the row will be selected.
			 * 
			 * @param  {integer} fromRow
			 * @param  {integer} fromCol
			 * @param  {boolean} reverseDirection if true, search backwards (up).
			 * @return {node}                  product node or null if not found.
			 */
			 _nextVerticalProductNodeInTable: function(fromRow, fromCol, reverseDirection) {
				if (!this._table || !this._table.firstChild) return null;
				var tbody = this._table.firstChild;
				var rows = tbody.rows;

				var increment = reverseDirection ? -1 : 1;
				var rowIndex = fromRow + increment;

				for (; rowIndex < rows.length && rowIndex >= 0; rowIndex += increment) {
					var cells = rows[rowIndex].cells;

					// First look left on this row
					for (var cellIndex = fromCol; cellIndex >= 0; cellIndex--) {
						var leftCell = cells[cellIndex];
						var productNode = dojo.query('.ple-HistoryGrid-product', leftCell)[0]; //$NON-NLS-1$
						if (productNode) return productNode;
					}

					// Now look right on this row
					for (cellIndex = fromCol; cellIndex < cells.length; cellIndex++) {
						var rightCell = cells[cellIndex];
						productNode = dojo.query('.ple-HistoryGrid-product', rightCell)[0]; //$NON-NLS-1$
						if (productNode) return productNode;
					}

				}

				return null;
			},

			/**
			 * Since focus cannot be looked up from the browser, we need to
			 * track the currently focus product node manually.
			 * @param  {event} e
			 */
			_onFocusNode: function(e) {
				this._lastFocusedProductNode = e.target;
			},

			/**
			 * Returns the coordinates of the last selected product node
			 * in the table.
			 * @return {object} null if no focused product. Otherwise, object
			 *     with .row and .cell properties.
			 */
			_getLastFocusedProductNodeCoords: function() {
				 if (!this._lastFocusedProductNode) return null;

				 var cell = closest(this._lastFocusedProductNode, 'TD'); //$NON-NLS-1$
				 var row = cell.parentNode;

				 return {
				 	row: row.rowIndex,
				 	cell: cell.cellIndex
				 };
			},

			/**
			 * Redraws the product node with a selected (orange) visual style.
			 * This include the arrows from the product to its successors. Does
			 * not change the actual selection--just changes styles.
			 * @param  {object} product JSON product to change style of.
			 * @param  {boolean} select  if true, draws as selected. If false, removes selected
			 *                           styles.
			 */
			_selectProductVisually: function(product, select) {
				if (product._arrowNodes) {
					dojo.forEach(product._arrowNodes, function(arrowNode) {
						arrowNode.removeShape();
					});

					delete product._arrowNodes;
				}

				if (select) {
					dojo.addClass(product.domNode, 'ple-HistoryGrid-productSelected'); //$NON-NLS-1$
					this._drawArrowsToSuccessors(product, true);
				}
				else {
					dojo.removeClass(product.domNode, 'ple-HistoryGrid-productSelected'); //$NON-NLS-1$
					this._drawArrowsToSuccessors(product, false);
				}
			},

			/**
			 * Clears all selected nodes. Changes their appearance to not be selected
			 * and clears the array of selected node.
			 * @return {[type]} [description]
			 */
			clearSelected: function() {
				while (this._selectedProducts.length) {
					var oldProduct = this._selectedProducts.pop();
					this._selectProductVisually(oldProduct, false);
				}			

				dojo.publish('ple.history.selectionDidChange', [this.getSelectedItemVersionUris()]);	 //$NON-NLS-1$
			},

			getSelectedItemVersions: function() {
				return this._selectedProducts;
			},

			/**
			 * Checks if a given item version URI is currently in the history grid.
			 * This is normally used to know if a change has been made to a visible item version and the
			 * grid should refresh.
			 * @return {boolean} is the item version is in the current history grid.
			 */
			isShowingItemVersionUri: function(itemVerisonUri) {
				return !!this._urisToProducts[itemVerisonUri];
			},

			getSelectedItemVersionUris: function() {
				return dojo.map(this._selectedProducts, function(product) {
					return product.uri;
				});
			},

			/**
			 * Deselects a product. Both changes it appearance and removes
			 * is from the selected array.
			 * @param  {object} product JSON product.
			 */
			_deselectProduct: function(product) {
				for (var i = 0; i < this._selectedProducts.length; i++) {
					var selectedProduct = this._selectedProducts[i];
					if (selectedProduct.uri === product.uri) {
						this._selectedProducts.splice(i, 1);
						this._selectProductVisually(product, false);
					}
				}
			},

			/**
			 * Event handle for when a product is clicked. Makes the product
			 * selected or unselected, depending on its current state, and if
			 * the user was holding the ctrl or shift key. Updates the array
			 * of selected products to match.
			 * @param  {DOM event} e 
			 */
			onClickProduct: function(e) {
				if (e.target.nodeName === 'A'){ //$NON-NLS-1$
					if (dojo.hasClass(e.target.parentNode, 'ple-HistoryGrid-missingSuccessor')) { //$NON-NLS-1$
						var node = new dojo.NodeList(e.target).closest('td').query('div.ple-HistoryGrid-product')[0]; //$NON-NLS-1$ //$NON-NLS-2$
						if (!node) return;
						dojo.stopEvent(e);

						var productUri = node['data-productUri']; //$NON-NLS-1$
						dojo.publish('ple.history.disableFilterForUri', [productUri]); //$NON-NLS-1$
						return;
					}
					else {
						// Normal links should pass through. The drop down menu is an a.button, so ignore that though.
						var isButton = e.target.role === 'button' || dojo.hasClass(e.target, 'button'); //$NON-NLS-1$ //$NON-NLS-2$
						if (!isButton) return;
					}
				} 


				// Delete the cached position. This is because the view may
				// have been scrolled at this point, and the old position is now wrong.
				// This is really only needed if we haven't already drawn all the arrows (due
				// to it taking too long in IE). Now we'll only draw them when a node is selected,
				// at which point the user may have already scrolled all around. We need to ensure
				// we get the position right at that moment in time.
				delete this._cachedWidgetPosition;

				var multiSelect = e.ctrlKey || e.shiftKey;
				
				var productNode = new dojo.NodeList(e.target).closest('div.ple-HistoryGrid-product')[0]; //$NON-NLS-1$
				if (!productNode) {
					return;
				}

				// Handling it here, so stop that event finally
				dojo.stopEvent(e);
				var product = this._urisToProducts[productNode['data-productUri']]; //$NON-NLS-1$

				this._selectProduct(product, multiSelect);

				if (e.type === 'contextmenu') { //$NON-NLS-1$
					this._openMenu(productNode, isRightClick(e), e);
				}
				else {
					// Don't focus if there's a menu open
					if (!dijit.popup.getTopPopup()) productNode.focus();
				}
			},

			/**
			 * Select a product in the view. Makes it orange and enables actions.
			 * @param  {string} productUri             the product to select.
			 * @param  {string} secondChoiceProductUri if the product isn't found, try to select this one. Used as a fallback if
			 * the first product is hidden by a filter.
			 */
			selectProductUri: function(productUri, secondChoiceProductUri) {
				var product = this._urisToProducts[productUri];
				if (!product) product = this._urisToProducts[secondChoiceProductUri];

				if (product) {
					this._selectProduct(product);
					return product.domNode;
				}
			},

			_selectProduct: function(product, multiSelect) {
				var alreadySelected = dojo.some(this._selectedProducts, function(oldSelectedProduct) {
					return oldSelectedProduct.uri == product.uri;
				});

				if (alreadySelected) {
					if (multiSelect) {
						this._deselectProduct(product);
						dojo.publish('ple.history.selectionDidChange', [this.getSelectedItemVersionUris()]); //$NON-NLS-1$
					}
					// else ignore, don't deselect an already selected product
				}
				else {
					if (!multiSelect) this.clearSelected();
					this._selectProductVisually(product, true);
					this._selectedProducts.push(product);
					dojo.publish('ple.history.selectionDidChange', [this.getSelectedItemVersionUris()]); //$NON-NLS-1$
				}			
			},

			/**
			 * Creates a parent-less DOM node which represents a single product
			 * (item version). This is visually a box with properties in it.
			 * @param  {object} product the JSON product.
			 * @return {DOM node}         the visual representation of a product.
			 */
			createProductNode: function(product) {
				var productNode = dojo.create('div', { //$NON-NLS-1$
					className: 'ple-HistoryGrid-product', //$NON-NLS-1$
					tabIndex: '0' //$NON-NLS-1$
				});

				dojo.create('div', { //$NON-NLS-1$
					className: "ple-HistoryGrid-product-title", //$NON-NLS-1$
					innerHTML: com.ibm.relm.ple.web.ui.formatting.formatProductIconAndTitleForItem(product)
				}, productNode);
				

				this._productNodeHandles.push(
					dojo.connect(productNode, 'onfocus', this, this._onFocusNode)); //$NON-NLS-1$

				var showMenuButton = new jazz.ui.toolbar.Button({
					imgClass: 'ple-icon-dropDownMenuWithArrow', //$NON-NLS-1$
					title: this.messages.actionsButton,
					onClick: dojo.hitch(this, this._openMenu, productNode, false)
				});

				showMenuButton.placeAt(productNode, 'first'); //$NON-NLS-1$

				this._buttons.push(showMenuButton);
				var audit = new com.ibm.relm.ple.web.ui.audit.ProductAudit({
					auditData: product._auditEntries,
					batchLoader: this._batchLoader
				});
				this._audits.push(audit);
				audit.placeAt(productNode);

				this._addToggleNodeDetailsButton(productNode);

				dojo.forEach(audit.resourceLinks, function(resourceLink) {
					this.connect(resourceLink, '_applyTitle', this._setNeedsArrowRedraw); //$NON-NLS-1$
				}, this);

				productNode['data-productUri'] = product.uri; //$NON-NLS-1$
				return productNode;
			},

			_addToggleNodeDetailsButton: function(productNode){
				var box = dojo.create('div', { //$NON-NLS-1$
					className: 'ple-HistoryGrid-toggleDetails' //$NON-NLS-1$
				});

				var productAuditNode = dojo.query('.ple-ProductAudit', productNode)[0]; //$NON-NLS-1$

				var toggleLink = dojo.create('a', { //$NON-NLS-1$
					href: '#', //$NON-NLS-1$
					className:"j-action-secondary" //$NON-NLS-1$
				});
				this._setToggleDetailsLinkText(toggleLink, productAuditNode);
				this._productNodeHandles.push(
					dojo.connect(toggleLink, 'onclick', this, this._toggleNodeDetails(productAuditNode))); //$NON-NLS-1$
				dojo.place(toggleLink, box);

				dojo.place(box, productNode, 'last'); //$NON-NLS-1$
			},

			//Returns the toggle event handler
			_toggleNodeDetails: function(productAuditNode){
				var self = this;
				return function(e){
					dojo.stopEvent(e);
					var toggleLink = e.target;

					if (!productAuditNode || !toggleLink)
						return;

					dojo.toggleClass(productAuditNode, 'ple-ProductAudit-collapsed'); //$NON-NLS-1$
					self._setToggleDetailsLinkText(toggleLink, productAuditNode);

					self._redrawAllArrows();
				};
			},

			_setToggleDetailsLinkText: function(toggleLink, productAuditNode){
				var audit = dijit.getEnclosingWidget(productAuditNode);
				var numDetails = audit.auditData.length;

				if (dojo.hasClass(productAuditNode, 'ple-ProductAudit-collapsed')){ //$NON-NLS-1$
					toggleLink.innerHTML = '<img src="' + this._blankGif + '" class="ple-icon-chevronDown">' + //$NON-NLS-1$ //$NON-NLS-2$
						(numDetails === 1 ? this.messages.show1Change : dojo.replace(this.messages.showXChanges, [numDetails]));
				} else {
					toggleLink.innerHTML = '<img alt="" src="' + this._blankGif + '" class="ple-icon-chevronUp">' + //$NON-NLS-1$ //$NON-NLS-2$
						(numDetails === 1 ? this.messages.hide1Change : dojo.replace(this.messages.hideXChanges, [numDetails]));
				}
			},

			hideAllDetails: function(hide){
				var productAuditNodes = dojo.query('.ple-ProductAudit', this.domNode); //$NON-NLS-1$

				var self = this;
				dojo.forEach(productAuditNodes, function(node){
					if (hide)
						dojo.addClass(node, 'ple-ProductAudit-collapsed'); //$NON-NLS-1$
					else 
						dojo.removeClass(node, 'ple-ProductAudit-collapsed'); //$NON-NLS-1$
					
					var toggleLink = dojo.query('.ple-HistoryGrid-toggleDetails a', node.parentElement)[0]; //$NON-NLS-1$
					self._setToggleDetailsLinkText(toggleLink, node);
				});

				this._redrawAllArrows();
			},

			_setNeedsArrowRedraw: function(t) {
				if (this._arrowRedrawTimeout) return;

				this._arrowRedrawTimeout = setTimeout(dojo.hitch(this, function() {
					this._redrawAllArrows();
					delete this._arrowRedrawTimeout;
				}), 0);
			},

			_newMenu: function() {
				var menu = new jazz.ui.Menu();

				var itemVersion = this.getSelectedItemVersions()[0];

				menu.addChild(new dijit.MenuItem({
					label: this.messages.viewInMyProducts,
					onClick: function() {
						dojo.publish('ple.history.viewInMyProducts'); //$NON-NLS-1$
					},
					iconClass: 'ple-icon-viewInMyProducts' //$NON-NLS-1$
				}));

				if (itemVersion && !itemVersion.isImmutable && Permissions.canProductsCreateSnapshot()) {
					menu.addChild(new dijit.MenuItem({
						label: this.messages.checkIn,
						onClick: function() {
							com.ibm.relm.ple.web.ui.productapi.checkInProduct(itemVersion);
						},
						iconClass: 'ple-icon-checkIn' //$NON-NLS-1$
					}));
				}

				if (Permissions.canProductsBranch()) {
					menu.addChild(new dijit.MenuItem({
						label: this.messages.createBranch,
						onClick: function() {
							dojo.publish('ple.history.createBranch'); //$NON-NLS-1$
						},
						iconClass: 'ple-icon-createBranch' //$NON-NLS-1$
					}));
				}

				menu.addChild(new dijit.MenuSeparator());

				menu.addChild(new dijit.MenuItem({
					label: this.messages.findUseLabel,
					onClick: function() {
						dojo.publish('ple.history.findUse'); //$NON-NLS-1$
					},
					iconClass: 'ple-icon-findUse' //$NON-NLS-1$
				}));

				if (Permissions.canProductsRedact()) {
					menu.addChild(new dijit.MenuSeparator());

					menu.addChild(new dijit.MenuItem({
						label: this.messages.redactVersionLabel,
						onClick: function() {
							dojo.publish('ple.history.redact'); //$NON-NLS-1$
						},
						iconClass: 'ple-icon-redact' //$NON-NLS-1$
					}));
				}

				return menu;
			},

			_openMenu: function(productNode, isRightClick, clickEvent) {
				dojo.addClass(productNode, 'ple-HistoryGrid-menuOpen'); //$NON-NLS-1$

				var menu = this._newMenu();

				var buttonNode = dojo.query('.jazz-ui-toolbar-Button', productNode)[0]; //$NON-NLS-1$

				var popupArgs = {
					parent: this,
					popup: menu,
					orient: {BL:'TL', BR: 'TR', TL:'BL', TR: 'BR'}, //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$ //$NON-NLS-4$
					onCancel: function() {
						dijit.popup.close(menu);
					},
					onClose: function() {
						dojo.removeClass(productNode, 'ple-HistoryGrid-menuOpen'); //$NON-NLS-1$
						menu.destroyRecursive();
						productNode.focus();
					},
					onExecute:  function(e) {
						dijit.popup.close(menu);
					}
				};

				if (isRightClick) {
					// Right click at point.
					popupArgs.x = clickEvent.clientX;
					popupArgs.y = clickEvent.clientY;
				}
				else {
					// Otherwise, from drop down menu button
					popupArgs.around = buttonNode;
				}

				dijit.popup.open(popupArgs);

				menu.connect(menu, 'onBlur', function() { //$NON-NLS-1$
					dijit.popup.close(menu);
				});

				menu.focus();
			},

			clear: function() {
				dojo.empty(this._table);
				this._arrowSurface && this._arrowSurface.clear();
			},

			/**
			 * Draws the full history grid from sctatch for the given
			 * history data. If there is already a drawn grid, erases the old one
			 * first.
			 * @param  {array} historyData history as an array of arrays. Ourter array
			 *                             represents the full rows. Inner arrays are the individual
			 *                             items in the row. Blank cell are represented as `null`.
			 *                             Filled cells have a JSON product in them, inlcuding references
			 *                             to their successors.
			 */
			draw: function(historyData) {
				this._batchLoader.reset();
				this._selectedProducts = [];
				this._urisToProducts = {};

				this._destroyAudits();
				this._destroyButtons();
				dojo.forEach(this._productNodeHandles, dojo.disconnect);
				this._productNodeHandles = [];
				dojo.empty(this._table);

				if (historyData.length > 0) {
					// A bit of a hack to get a fixed size parent for the table,
					// ensuring the table won't resize. This should be accomplished with 
					// table-layout: fixed, but that doesn't seem to work since the scroll region
					// containing this is absolute.
					var numberOfColumns = historyData[0].length;
					this.domNode.style.width = numberOfColumns * BRANCH_COLUMN_CSS_PX + 'px'; //$NON-NLS-1$
				}

				var tbody = dojo.create('tbody'); //$NON-NLS-1$

				for (var i = 0; i < historyData.length; i++) {
					var rowData = historyData[i];
					var tr = dojo.create('tr', null, tbody); //$NON-NLS-1$
					var matrixRow = [];
					tbody.appendChild(tr);

					for (var j = 0; j < rowData.length; j++) {
						var productData = rowData[j];

						var td = dojo.create('td', { //$NON-NLS-1$
							className: 'ple-HistoryGrid-gridCell' //$NON-NLS-1$
						}, tr);

						if (!productData) {
							matrixRow.push(null);
						}
						else {
							// It's OK for a product to be 'null' here, which just indicates
							// there is no product in this slot of this history. Still need the table cell
							// for layout puposes though.
							this._urisToProducts[productData.uri] = productData;
							var productNode = this.createProductNode(productData);
							matrixRow.push(productData);

							td.appendChild(productNode);
							productData.domNode = productNode;
						}
					}
				}

				this._drawMissingSuccessorBoxes();

				this._table.appendChild(tbody);

				this._drawAllArrows();
			},

			_newMissingSuccessorBox: function(missingCount) {
				var box = dojo.create('div', { //$NON-NLS-1$
					className: 'ple-HistoryGrid-missingSuccessor' //$NON-NLS-1$
				});

				var m = this.messages;
				var linkLabel = missingCount === 1 ?
					dojo.replace(m.oneHiddenBranchesLink, [missingCount]) :
					dojo.replace(m.xNumberHiddenBranchesLink, [missingCount]);
				box.innerHTML = dojo.replace('<a class="j-action-secondary" href="#" title="{1}">{0}</a>', //$NON-NLS-1$
					[linkLabel, m.hiddenBranchesTooltip]);
				return box;
			},

			_drawMissingSuccessorBoxes: function() {
				for (var uri in this._urisToProducts) {
					var product = this._urisToProducts[uri];
					var successors = product._successors;
					if (!successors) continue;
					
					var missingCount = 0;

					dojo.forEach(successors, function(successorUri) {
						if (!this._urisToProducts[successorUri]) missingCount += 1;
					}, this);

					if (missingCount > 0) {
						var box = this._newMissingSuccessorBox(missingCount);
						dojo.place(box, product.domNode.parentNode, 'first'); //$NON-NLS-1$
					}
				}

			},

			onArrowsDidDraw: function() {
				
			},

			getAllPositions: function() {
				var layout = {
					containerPosition: this._getWidgetPosition()
				};

				layout.memberPositions = [];

				for (var id in this._urisToProducts) {
					var product = this._urisToProducts[id];
					var pos = dojo.clone(this._getCachedProductPosition(product));
					pos.uri = id;
					layout.memberPositions.push(pos);
				}

				return layout;
			},

			/**
			 * Given a product that already has a .domNode property
			 * associated with it, returns the absolute page position
			 * of the DOM node. If the position has already been determined, then
			 * the cached value is used. This means a product cannot be moved,
			 * or the position is wrong in the cache. In practice, the whole grid is redrawn
			 * when there is a change though.
			 * @param {object} productData the JSON product with a .domNode property (added elsewhere in 
			 *                             this class).	
			 * @return {[type]}            dojo.position for the product node.
			 */
			_getCachedProductPosition: function(productData) {
				if (productData._position) return productData._position;
				var position = dojo.position(productData.domNode, true);


				// dojo.position is relative to page. We want them relative
				// to this widget, so take the difference.
				var widgetPosition = this._getWidgetPosition();
				position.x -= widgetPosition.x;
				position.y -= widgetPosition.y;

				var parent = this.domNode.parentNode;
				position.x += parent.scrollLeft;
				position.y += parent.scrollTop;

				return productData._position = position;
			},

			/**
			 * Draws a single line witharrow head on the arrow surface.
			 * 	
			 * @param  {object} start start point of the line.
			 * @param  {object} end   end point of the line.
			 * @param {boolean} selected true to draw with selected visual style.
			 * @return {object}	the new dojo.gfx group for the arrow (which is made of multiple paths).
			 */	
			_drawArrow: function(start, end, selected) {
				var blue = '#2e5589'; //$NON-NLS-1$
				var orange = '#fc540c'; //$NON-NLS-1$

				var color = selected ? orange : blue;

				// draw on .5 px boundaries. This makes for cleaner lines since 
				// a 1px line extends .5 px on both sides. Starting on .5 means
				// the left and right edges of the line fall on 2hole pixel boundaries for 
				// straight lines.
				
				var g = this._arrowSurface.createGroup();

				// Start with initial, straight line.
				// End it below the end point, so that there's room for an arrowhead.
				// Draw an orange outline if necessary
				g.createPath().
					moveTo(start.x + 0.5, start.y).
					lineTo(end.x + 0.5, end.y).
					setStroke({width: 1, color: color});

				// Now add arrowhead.
				// The arrow is drawn pointing left-to-right, with the point at 0, 0, and rotated to 
				// match the line. The actually seems like it's the opposite direction it should be,
				// but HTML coordinates are not cartesian: +y moves things down. I'm not clear
				// on why this direction matches the angle properly, but it does. :)
				g.createPath().
					moveTo(end.x + 0.5, end.y).
					setTransform(dojox.gfx.matrix.rotateAt(angle(start, end), end.x + 0.5, end.y)).
					lineTo(end.x + 0.5 - 15, end.y - 5).
					lineTo(end.x + 0.5 - 13, end.y).
					lineTo(end.x + 0.5 - 15, end.y + 5).
					setFill(color);

				return g;
			},

			/**
			 * Draws arrows to all the successors of a product.
			 * @param  {object} productData the product to draw from.
			 * @param {boolean} selected true to draw selected visual style.
			 */
			_drawArrowsToSuccessors: function(productData, selected) {
				if (!productData._successors || productData._successors.length === 0) return;

				var startPosition = this._getCachedProductPosition(productData);

				var arrows = [];

				for (var i = 0; i < productData._successors.length; i++) {
					var successorId = productData._successors[i];
					var successor = this._urisToProducts[successorId];
					if (!successor) {
						// Normal if a filter is on (we didn't create nodes for them since they are hidden).
						continue;
					}
					var endPosition = this._getCachedProductPosition(successor);

					var arrowNode = this._drawArrow(topCenter(startPosition), bottomCenter(endPosition), selected);
					arrows.push(arrowNode);
				}

				productData._arrowNodes = arrows;
			},

			/**
			 * We want the x and y of this whole widget, to position things relative
			 * to it. But, the only children are absolutely position, meaning this.domNode
			 * has an inaccurate width (viewport size). We really want the width of the table
			 * since the arrow surface needs to be the same size.
			 * @return {[type]} [description]
			 */
			_calculateWidgetPosition: function() {
				var widgetPosition = dojo.position(this.domNode, true);
				var tablePosition = dojo.position(this._table, true);
				widgetPosition.w = tablePosition.w;
				widgetPosition.h = tablePosition.h;

				var parent = this.domNode.parentNode;
				widgetPosition.x += parent.scrollLeft;
				widgetPosition.y += parent.scrollTop;

				return widgetPosition;
			},

			/**
			 * Gets this absolute position of this widget on the page.
			 * Returns the cached position if possible, making this method very fast.
			 * @param  {boolean} clearCache true to clear the cached position an ensure the
			 *                              current position is returned. Not for use in loops, but needed up front on 
			 *                              new layouts
			 */
			_getWidgetPosition: function(clearCache) {
				if (clearCache || !this._cachedWidgetPosition) this._cachedWidgetPosition = this._calculateWidgetPosition();
				return this._cachedWidgetPosition;
			},

			_redrawAllArrows: function() {
				delete this._cachedWidgetPosition;

				for (var uri in this._urisToProducts) {
					delete this._urisToProducts[uri]._position;
				}

				this._drawAllArrows();

				// It's faster to just redraw the selected arrows after the fact instead of checking all the node
				// whiel redrawing them. Without this, we'd lose the orange arrow of the selected product.
				for (var i = 0; i < this._selectedProducts.length; i++) {
					var product = this._selectedProducts[i];
					this._selectProductVisually(product, true);
				}
			},

			/**
			 * Creates a the arrow "surface" and draws arrows from ever product
			 * to their successors.
			 * @param  {[type]} historyData all the history data. Array of array. Rows for product.
			 */
			_drawAllArrows: function() {
				// don't double draw
				clearTimeout(this._lastResortDrawArrowTimeout);

				// Make sure we clear the position cache at first.
				var widgetPosition = this._getWidgetPosition(true);

				if (this._arrowSurface) {
					// Destroy and recreate so we can make one with a new size.
					dojo.destroy(this._arrowSurface.rawNode);
					this._arrowSurface.destroy();
					delete this._arrowSurface;
				}

				// Create the Surface for the arrows to draw in
				this._arrowSurface = dojox.gfx.createSurface(
					dojo.create('div'),	// dummy DOM node since we may recreate this later. //$NON-NLS-1$
					widgetPosition.w,
					widgetPosition.h);

				// Add to the top of this widget to be aligned with the table/grid.
				// IE insists that this is in the document DOM before drawing to it, 
				// so it must be added before the arrows.
				dojo.place(this._arrowSurface.rawNode, this.domNode);

				dojo.style(this._arrowSurface.rawNode, {
					position: 'absolute', //$NON-NLS-1$
					top: '0', //$NON-NLS-1$
					left: '0', //$NON-NLS-1$
					zIndex: 0
				});

				var i = 0;
				var startTime = new Date().getTime();
				// Iterate over all products and successors to draw their arrows

				for (var uri in this._urisToProducts) {
					var productData = this._urisToProducts[uri];
					this._drawArrowsToSuccessors(productData);

					if (i % 10 === 0) {
						if (new Date().getTime() - startTime > MAX_DRAW_TIME_MILLIS) {
							dojo.publish('ple.history.arrowsTooSlow'); //$NON-NLS-1$
							break;
						}
					}

					i++;
				}

				
				this.onArrowsDidDraw(this.getAllPositions());
			},

			uninitialize: function() {
				this._selectedProducts = null;
				this._urisToProducts = null;
				this._arrowSurface && this._arrowSurface.destroy();
				this._destroyAudits();
				this._destroyButtons();	
				dojo.forEach(this._productNodeHandles, dojo.disconnect);
				this._batchLoader.reset();

				this.inherited(arguments);
			}
		}
	);

})();	
