/*******************************************************************************
 * Licensed Materials - Property of IBM
 * Copyright IBM Corp. 2011, 2012.  All Rights Reserved.
 *
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *******************************************************************************/

/**
 * Provides common utility operations for working in the design page, such as
 * presentation URLs, and facilities for registering actions that can be
 * displayed in the explorer.
 * 
 * @author Adam R. Neal
 */
(function() {
dojo.provide("rmps.app.ui.internal.pages.design.DesignUtil");

var designPrefix = "rmps.app.design.";

rmps.app.ui.internal.pages.design.DesignUtil = {

	createMode: 'create',
	editMode: 'edit',
	exploreMode: 'explore',
	importMode: 'importMetamodel',
	importOwlMode: 'importOwl',
	validationMode: 'validation',
	queryMode: 'query',
	queriesMode: 'queries',
	viewpointMode: 'viewpoint',
	viewpointsMode: 'viewpoints',
	clipboardMode: 'clipboard',
	
	actionPrefix: designPrefix,

	// Defined in __init
	createAction: null,
	exploreAction: null,
	editAction: null,
	defaultAction: null,
	importAction: null,
	importOwlAction: null,
	queryAction: null,
	queriesAction: null,
	viewpointAction: null,
	viewpointsAction: null,
	clipboardAction: null,
	validationAction: null,

	_basicPrefix: "#action=",
	_explorerActions: {},  // map of uri/type to action
	_avoidEdits: {}, // map of URIs that should not show the edit menu
	
//	getWelcomeUri: function() {
//		return this._basicPrefix + "welcome";
//	},

	_createURI: function (resourceUri, action) {
		var result = this._basicPrefix + action;
		
		if (resourceUri) {
			result += '&' + dojo.objectToQuery({"resource": resourceUri});
		}
		return result;
	},
	
	getCreateResourceUri: function(typeUri) {
		return this._createURI(typeUri, this.createAction);
	},
	
	getValidationResourceUri: function(resourceUri) {
		return this._createURI(resourceUri, this.validationAction);
	},
	
	getEditResourceUri: function(/*String*/resourceUri, /*String?*/tab)  {
		// Create the URL to take the user into edit mode of the given resource
		// 		param resourceUri:   the URI of the resource to edit
		//		param tab:			 optional, the specific tab to go to (no checking is done though to ensure this is valid!)
		var uri = this._createURI(resourceUri, this.editAction);
		if (tab && typeof tab === 'string') {
			uri += "&" + dojo.objectToQuery({"tab": tab});
		}
		return uri;
	},
	
	getExploreResourceUri: function(resourceUri)  {
		return this._createURI(resourceUri, this.exploreAction);
	},
	
	getImportResourceUri: function(resourceUri)  {
		return this._createURI(resourceUri, this.importAction);
	},	
	getOwlImportResourceUri: function(resourceUri)  {
		return this._createURI(resourceUri, this.importOwlAction);
	},	
	getQueryUri: function(resourceUri) {
		return this._createURI(resourceUri, this.queryAction);
	},

	getViewpointUri: function(resourceUri) {
		return this._createURI(resourceUri, this.viewpointAction);
	},
	
	getQueriesUri: function() {
		return this._createURI(null, this.queriesAction);
	},
		
	getViewpointsUri: function() {
		return this._createURI(null, this.viewpointsAction);
	},
	
	getClipboardUri: function() {
		return this._createURI(null, this.clipboardAction);
	},
	
	registerExploreAction: function(/*String*/key, /*Object|Array*/data) {
		/*
		 * This method will register a (set of) actions with a given URI/Type so that the
		 * explorer can provider domain specific content actions.  This method can be called
		 * multiple times with the same key, data will simply be appended.
		 * 
		 * @param key - expected to be URI or Type that this action is associated with
		 * @param data - Object of or an array of objects contains either
		 * { label:String,        		visible name of action  
		 *   iconClass:String,    		icon class to use
		 *   callback(editor):function  the action to perform, will receive the editor as a param 
		 * }
		 * OR to identify a separator:
		 * {
		 *   isSeparator:bool
		 * }
		 * 
		 * 
		 */
		
		var entry = this.getExplorerAction(key) || [];
		
		if (dojo.isArray(data)) {
			entry = entry.concat(data);
		} else if (dojo.isObject(data)){
			entry.push(data);	
		}
		
		this._explorerActions[key] = entry; 
	},
	
	getExplorerAction: function(/*String*/key) {
		/*
		 * @param key
		 * 				expected to be a URI or Type
		 * returns an array of action objects or <code>undefined</code>
		 * 
		 * returned object contains: label, icon, callback
		 */
		return this._explorerActions[key];
	},
	
	avoidEdit: function(uri) {
		return !!this._avoidEdits[uri];
		// TODO Server should provide...
		return ("http://ibm.com/dtk/DomainConceptsNode" === uri);
	},
	
	registerUriToAvoidEdit: function(uri, unset) {
		/*
		 * Domain specific design pages should register their
		 * top level nodes with this API in order to avoid the edit
		 * menu showing up.
		 * Usage:  DesignUtil.registerUriToAvoidEdit(myURI);
		 * 
		 * or, in the off chance that you want to unregister it:
		 * DesignUtil.registerUriToAvoidEdit(myURI, false);
		 */
		this._avoidEdits[uri] = unset === undefined || !!unset;
	}
};

// Initialize some variables
(function __init (util) {
	util.createAction = designPrefix + util.createMode;
	util.editAction = designPrefix + util.editMode;
	util.exploreAction = designPrefix + util.exploreMode;
	util.importAction = designPrefix + util.importMode;
	util.importOwlAction = designPrefix + util.importOwlMode;
	util.validationAction = designPrefix + util.validationMode;
	util.queryAction = designPrefix + util.queryMode;
	util.queriesAction = designPrefix + util.queriesMode;	
	util.viewpointAction = designPrefix + util.viewpointMode;
	util.viewpointsAction = designPrefix + util.viewpointsMode;
	util.clipboardAction = designPrefix + util.clipboardMode;
	util.defaultAction = util.exploreAction;
}(rmps.app.ui.internal.pages.design.DesignUtil));

})();


