/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2014. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:  
 * Use, duplication or disclosure restricted by GSA ADP Schedule 
 * Contract with IBM Corp. 
 *******************************************************************************/
dojo.provide("com.ibm.relm.explorer.web.ui.internal.view.edit.LinkPathFinder"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils"); //$NON-NLS-1$
dojo.require("com.ibm.relm.explorer.web.ui.internal.view.common.RequestQueue"); //$NON-NLS-1$

(function(){
	var editorUtils = com.ibm.relm.explorer.web.ui.internal.editor.EditorUtils;

	dojo.declare("com.ibm.relm.explorer.web.ui.internal.view.edit.LinkPathFinder", null, { //$NON-NLS-1$
		viewer: null,
		source: null,
		target: null,
		_linkpath: null,
		_canceled: false,
		
		constructor : function(args) {
			dojo.mixin(this, args);
			this.restart();
		},
		
		restart: function() {
			this.cancel();
			this._canceled = false;
			this._linkpath = [];
		},
		
		cancel: function() {
			this._canceled = true;
		},
		
		destroy: function() {
			this.cancel();
			this._linkpath = null;
			this.inherited(arguments);
		},
		
		/**
		 * Get list of link types that can be used between source and target containers
		 */
		getPossibleLinkPath: function(callback, cancelcallback, start, end) {
			var endDepth = end ? end : -1;

			if (this.source.elementId && this.target.elementId) {
				var sourceId = this.source.elementId;
				var targetId = this.target.elementId;

				var sourceNode = {id: sourceId};
				this.viewer.getArtifactElementTitle(sourceId, function(title) { sourceNode.title = title; });

				var parameters = {depth: 0, executionInfos: [{history: [], parentPath: [sourceNode], params: {sourceId: sourceId, targetId: targetId}}]};
				this._search(parameters, function(linkpath, progress) {
					if (progress == -2) {
						cancelcallback && cancelcallback();
					} else if (progress >= endDepth || progress < 0) {
						callback(linkpath, progress);
					}
				});
			} else {
				editorUtils.getConnectionTypeItems(dojo.hitch(this, function(linkTypes){
					var linkpath = [];
					dojo.forEach(linkTypes, function(link) {
						linkpath.push([{id: this.target.getId(), label: link.name, linktype: link.value}]);
					}, this);
					callback(linkpath, -1);
				}), true);
			}
		},
	
		/**
		 * Search link path
		 */
		_search: function(parameters, callback) {
			var deferreds = [];
	
			var self = this;
			var targetId = this.target.elementId;
			var depth = parameters.depth;
			var executionInfos = parameters.executionInfos || [];
				
			var childParameters = {depth: depth + 1, executionInfos: []};

			// request queue to serialize search operations of linkpaths
			var requestQueue = new com.ibm.relm.explorer.web.ui.internal.view.common.RequestQueue();

			while(executionInfos.length > 0) {
				var infos = executionInfos.splice(0, 100);
		
				deferreds.push(requestQueue.request({
					params: {depth: depth, executionInfos: infos}, 
					func: function(requestParams) {
						var operations = [];
						// Firstly, collect all linkpaths in this depth
						dojo.forEach(requestParams.executionInfos, function(info) {
							var history = info.history;
							var parentPath = info.parentPath;
							var params = info.params;

							if (self._canceled) {
								requestQueue.cancelAll();
								return;
							}

							var ret = new dojo.Deferred();
							operations.push(ret);
							
							self._findPossibleLinkPaths(params, history, parentPath, depth).then(function(result) {
								if (self._canceled) {
									ret.callback();
									requestQueue.cancelAll();
									return;
								}
								
								if (result) {
									var executionInfos = dojo.map(result.connections, function(connection) {
										var parentPath = dojo.clone(result.parentPath);
										parentPath.push(connection);
										return {history: result.history, parentPath: parentPath, params: {sourceId: connection.id, targetId: targetId}};
									});
									childParameters.executionInfos = childParameters.executionInfos.concat(executionInfos);
									ret.callback(executionInfos);
								} else {
									ret.callback();
								}
							});			
						});
						return dojo.when(new dojo.DeferredList(operations));
					}}));
			}

			// Secondly, request to search one level deeper depth
			dojo.when(new dojo.DeferredList(deferreds), function() {
				if (self._canceled) {
					callback(self._linkpath, -2);
					return;
				}

				callback(self._linkpath, depth);
				if (childParameters.executionInfos.length == 0) {
					callback(self._linkpath, -1);
				} else {
					setTimeout(function() {
						if (self._canceled) {
							callback(self._linkpath, -2);
							return;
						}
						self._search(childParameters, callback);
					}, 50);
				}
			});
		},
		
		/**
		 * Find linkpaths of certain depth
		 */
		_findPossibleLinkPaths: function(params, pathHistory, parentPath, depth) {
			var deferred = new dojo.Deferred();

			var self = this;
			this.viewer.getArtifactElement(params.sourceId, function(element) {
				if (!element.properties || element.properties.length == 0) {
					// no link information
					deferred.callback();
					return;
				}

				var startDepth = params.startDepth ? params.startDepth : 0;
				var endDepth = params.endDepth ? params.endDepth : -1;
			
				var connections = null;
				dojo.forEach(element.properties, function(property) {
					dojo.forEach(property.linkTo, function(linkTo) {
						var connection = {id: linkTo.id, label: linkTo.label, title: linkTo.label, sourceProperty: property.name};
						self.viewer.getArtifactElementTitle(linkTo.id, function(title) {
							connection.title += " [" + title + "]"; //$NON-NLS-2$ //$NON-NLS-1$
							connection.target = title;
						});
						if (depth >= startDepth && linkTo.id == params.targetId) {
							// linkpath found, add to list
							var path = dojo.clone(parentPath);
							path.push(connection);
							if (self._linkpath) self._linkpath.push(path);
							else self._linkpath = path;
						} else if (params.sourceId == linkTo.id) {
								// prevent infinite loop
							} else if (dojo.indexOf(pathHistory, linkTo.id) >= 0) {
								// prevent infinite loop
						} else if (endDepth < 0 || depth < endDepth) {
							// save links to search hierarchically
							if (!connections) connections = [];
							connections.push(connection);
						}
					});
				});
				
				if (connections) {
					// return result
					var history = pathHistory ? dojo.clone(pathHistory) : [];
					history.push(params.sourceId);
					var result = {history: history, parentPath: parentPath, connections: connections};
					deferred.callback(result);
				} else {
					deferred.callback();
				}
			}, function(error) {
				deferred.callback();
			});
			return deferred;
		}
});
})();
