/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.relm.setup.ui.internal.wizard.Util"); //$NON-NLS-1$

dojo.require("dojox.xml.parser"); //$NON-NLS-1$
dojo.require("jazz.ui.MessageBox"); //$NON-NLS-1$
dojo.require("jazz.util.html"); //$NON-NLS-1$

dojo.require("dojo.i18n"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.relm.setup.ui.internal.wizard", "WizardMessages"); //$NON-NLS-1$ //$NON-NLS-2$

/**
 * Common wizard utility functions that don't have a better home.
 */
(function(){
var STATUS_TYPES = jazz.ui.MessageBox.STATUS_TYPES;
var messages = dojo.i18n.getLocalization("com.ibm.relm.setup.ui.internal.wizard", "WizardMessages"); //$NON-NLS-1$ //$NON-NLS-2$

/*
 * Build up the arguments to a MessageBox from the error information.
 * Attempts to interrogate the response since the content-type varies
 * according to the call performed.
 * 
 * @param error The error object of the error handler.
 * @param ioArgs The ioArgs from the error handler.
 * @param dfltSummary Used as a fallback if a more specific message can't be determined.
 */
com.ibm.relm.setup.ui.internal.wizard.Util.buildMessageBoxArgs = function(error, ioArgs, dfltSummary) {
	var messageBoxArgs = {
		type: STATUS_TYPES.ERROR,
		summary: dfltSummary,
		errorObject: error
	};
	
	var contentType = ioArgs.xhr.getResponseHeader("Content-Type"); //$NON-NLS-1$
	if (!contentType) contentType = ""; //$NON-NLS-1$
	
	if (contentType.indexOf("application/rdf+xml") != -1) { //$NON-NLS-1$
		try {
			var doc = dojox.xml.parser.parse(error.responseText);
			var docElem = doc.documentElement;
			var JAZZ_NS = "http://jazz.net/xmlns/prod/jazz/foundation/1.0/"; //$NON-NLS-1$
			
			var errorMessageElem = docElem.getElementsByTagNameNS(JAZZ_NS, "errorMessage")[0]; //$NON-NLS-1$
			var errorMessage = errorMessageElem.firstChild.nodeValue;
			var detailedMessageElem = docElem.getElementsByTagNameNS(JAZZ_NS, "detailedMessage")[0]; //$NON-NLS-1$
			var detailedMessage = detailedMessageElem.firstChild.nodeValue;
			
			messageBoxArgs.summary = errorMessage;
			messageBoxArgs.details = [{message: detailedMessage, code: STATUS_TYPES.ERROR}];
		} catch (err) {
			console.debug("Could not process XML response: ", err); //$NON-NLS-1$
			console.debug("error.responseText: ", error.responseText); //$NON-NLS-1$
		}
	}
	else if (contentType.indexOf("application/json") != -1) { //$NON-NLS-1$
		var mbe = null;
		
		try {
			mbe = dojo.fromJson(error.responseText);
		}
		catch (err) {
			console.debug("Could not parse JSON object: ", err); //$NON-NLS-1$
			console.error("Invalid JSON: ", error.responseText); //$NON-NLS-1$
			return messageBoxArgs;
		}
		
		var details = [];
		
		if (mbe.summary) messageBoxArgs.summary = mbe.summary;
		if (mbe.explanation) details.push({message: mbe.explanation, code: STATUS_TYPES.ERROR});
		if (mbe.userAction) details.push({message: mbe.userAction, code: STATUS_TYPES.ERROR});
		if (mbe.stackTrace) details.push({message: mbe.stackTrace, code: STATUS_TYPES.ERROR});
		if (details.length > 0) messageBoxArgs.details = details;
	}
	else {
		messageBoxArgs.details = [{message: error.responseText, code: STATUS_TYPES.ERROR}];
	}
	
	return messageBoxArgs;
};
	
/**
 * The /relm/configure POST operations don't directly report their status
 * details, instead returning a requestId. Follow up with a GET for status
 * information as a JSON response:
 * <code>
 * {
 * 	"error": ["Error 1 occurred.", "Error 2 occurred."],
 * 	"warning": []
 * }
 * </code>
 */
com.ibm.relm.setup.ui.internal.wizard.Util.getAndProcessRequestDetails = function(requestId, domNode) {
	var query = {
		"requestId": requestId, //$NON-NLS-1$
		"clear": true //$NON-NLS-1$
	};
	
	var showInlineStatus = com.ibm.relm.setup.ui.internal.wizard.Util.showInlineStatus;
	
	jazz.client.xhrGet({
		url: net.jazz.ajax._contextRoot + "/configure?" + dojo.objectToQuery(query), //$NON-NLS-1$
		handleAs: "json", //$NON-NLS-1$
  		headers: {
			"X-ibm-rmps-internal": "true" //$NON-NLS-1$ //$NON-NLS-2$
		},

		load: function(response, ioArgs) {
			try {
				var errors = response.error;
				var warnings = response.warning;
				
				if (errors.length > 0) {
		        	var messageBoxArgs = {
		        			type: STATUS_TYPES.ERROR,
		        			summary: formatMessageArray(errors)
		        	};
		        	
		        	showInlineStatus(domNode, messageBoxArgs);
				}
				else if (warnings.length > 0) {
		        	var messageBoxArgs = {
		        			type: STATUS_TYPES.WARNING,
		        			summary: formatMessageArray(warnings)
		        	};
		        	
		        	showInlineStatus(domNode, messageBoxArgs);
				}
				else { // no news is good news!
					dojo.removeClass(domNode, "loading"); //$NON-NLS-1$
					dojo.addClass(domNode, "ok"); //$NON-NLS-1$
				}
			}
			catch (err) { // JSON response object didn't match expected shape
				console.debug(err);
				console.warn("JSON response object didn't match expected shape: ", response); //$NON-NLS-1$
				
	        	var messageBoxArgs = {
	        			type: STATUS_TYPES.WARNING,
	        			summary: messages.indeterminateStatus,
	        			details: [ {message: err.toString() } ]
	        	};
	        	
	        	showInlineStatus(domNode, messageBoxArgs);
			}
        },

        error: function(error, ioArgs) {
        	// Although an error occurred, it was with fetching the status of the operation,
        	// not the operation itself. Hence the warning.
        	var messageBoxArgs = {
        			type: STATUS_TYPES.WARNING,
        			summary: messages.indeterminateStatus,
        			details: [ {message: jazz.util.html.escape(error.responseText)} ],
        			errorObject: error
        	};
        	
        	showInlineStatus(domNode, messageBoxArgs);
        }
	});
};
	
/**
 * Show a status message for a wizard operation in it's inline MessageBox.
 * 
 * @param node An operation node (represents a wizard operation).
 * @param messageBoxArgs The arguments for MessageBox.showMessage()
 * 			{@see https://jazz.net/wiki/bin/view/WebUIBook/MessageBox}.
 */
com.ibm.relm.setup.ui.internal.wizard.Util.showInlineStatus = function(node, messageBoxArgs) {
	var status = messageBoxArgs.type ? messageBoxArgs.type.toLowerCase() : STATUS_TYPES.ERROR;
	dojo.removeClass(node, "loading"); //$NON-NLS-1$
	dojo.addClass(node, status);
	var errorDiv = node.getElementsByTagName("div"); //$NON-NLS-1$
	dojo.removeClass(errorDiv[0], "noDisplay"); //$NON-NLS-1$
	node._errorMsgBox.showMessage(messageBoxArgs);
};
	
/**
 * Break up an array of messages (errors or warnings) into
 * escaped, line-separated messages suitable for a MessageBox.
 * 
 * @param msgs The array of messages.
 */
formatMessageArray = function(msgs) {
	var all = ""; //$NON-NLS-1$
	
	for (var i = 0; i < msgs.length; i++) {
		all += jazz.util.html.escape(msgs[i]);
		if (i < msgs.length - 1)
			all += "<br>"; //$NON-NLS-1$
	}
	
	return all;
};

})();