/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.picker.ColumnBrowser'); //$NON-NLS-1$

dojo.require('dijit._Templated'); //$NON-NLS-1$
dojo.require('dojo.fx.easing'); //$NON-NLS-1$

(function(){

	var SLIDE_DURATION = 300;
	var EASING = dojo.fx.easing.quadIn;

	/**
	 * Allows several widgets to be added to a stack. Only one is visible
	 * at a time, and it will be modified to fill the entire space that this
	 * column browser widget fills (this widget has no visual representation
	 * beyond clipping nested widgets to it bounds).
	 *
	 * When a new widget is pushed on, the old widget animates to the left
	 * and the new widget slide in from the right, similar to what's seen
	 * commonly on mobile phones. When a widget is popped off, the opposite
	 * animation is shown.
	 *
	 * This was added to support selecting "nested" items from the LQE artifact picker,
	 * specifically DOORS baselines nested under DOORS modules.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.picker.ColumnBrowser', //$NON-NLS-1$
		[ dijit._Widget, dijit._Templated ],
		{
			templateString: '<div class="ple-ColumnBrowser"></div>', //$NON-NLS-1$
			widgetsInTemplate: false,

			/**
			 * Returns the top (only visible) widget on the stack.
			 * Null if there is none.
			 * @type {object} any top widget or null.
			 */
			 topWidget: function() {
			 	var len = this._widgetStack.length;
			 	return this._widgetStack[len - 1];			 	
			 },
			
			constructor: function(args, node)
			{
				this._widgetStack = [];
			},

			/**
			 * Pushes a widget onto the stack and shows it at the same size
			 * as this this. If it's the first widget, it shows immediately.
			 * Later widgets are animated in.
			 *
			 * Any widgets on the stack when this widget is destroyed
			 * will also be destroyed.
			 * 
			 * @param  {Widget} widget the widget to display.
			 * @return {Deferred} fires when animation is complete.
			 */
			pushWidget: function(widget) {
				var thisPos = dojo.position(this.domNode);

				var oldTop = this.topWidget();

				this._widgetStack.push(widget);

				if (oldTop) {
					var outAnim = dojo.animateProperty({
						node: oldTop.domNode,
						duration: SLIDE_DURATION,
						easing: EASING,
						properties: {
							left: {
								start: 0,
								end: -thisPos.w
							}
						},
						onEnd: function() {
							oldTop.domNode.style.display = 'none'; //$NON-NLS-1$
						}
					}).play();
				}

				// Force this widget to fill to column browser size.
				// The widget is respsonsible for its own scrolling if needed.
				dojo.style(widget.domNode, {
					position: 'absolute', //$NON-NLS-1$
					top: 0,
					left: 0,
					boxSizing: 'border-box', //$NON-NLS-1$
					width: thisPos.w + 'px', //$NON-NLS-1$
					width: '100%', //$NON-NLS-1$
					height: '100%' //$NON-NLS-1$
				});

				dojo.place(widget.domNode, this.domNode);

				var deferred = new dojo.Deferred();
				if (oldTop) {
					var inAnim = dojo.animateProperty({
						node: widget.domNode,
						duration: SLIDE_DURATION,
						easing: EASING,
						properties: {
							left: {
								start: thisPos.w + 10,
								end: 0
							}
						},
						onEnd: function() {
							deferred.callback();
						}
					}).play();
				}

				return deferred;
			},

			/**
			 * Pops all the widgets off the stack except for the first one.
			 * This is an easy way to "reset" a result list. Used in the LQE
			 * artifact picker when a new search starts.
			 * 
			 * @return {dojo.deferred} fires once every widget (except the first)
			 *                               has been popped, that is, after the animation are done.
			 *                               The called **must** destroy the popped widgets on its own.
			 */
			popAllWidgetsExceptFirst: function() {
				var deferred = new dojo.Deferred();
				var allPoppedWidgets = [];

				var self = this;
				function popNext() {
					if (self._widgetStack.length < 2) {
						deferred.callback(allPoppedWidgets);
						return;
					}

					self.popWidget().then(function(poppedWidget) {
						allPoppedWidgets.push(poppedWidget);
						popNext();
					});
				}

				popNext();
				return deferred;
			},

			/**
			 * Pops the top widget and slides in the previous one.
			 * The popped widget will no longer be owned by this one, and
			 * must be destroyed somewhere else.
			 * 
			 * @return {Deferred} deferred that fires once the popped widget
			 * is done animating out. Should be used to clean up the widget as needed.
			 */
			popWidget: function() {
				var deferred = new dojo.Deferred();

				var thisPos = dojo.position(this.domNode);
				var tailWidget = this._widgetStack.pop();

				if (!tailWidget) {
					deferred.callback();
				}
				else {
					var outAnim = dojo.animateProperty({
						node: tailWidget.domNode,
						duration: SLIDE_DURATION,
						easing: EASING,
						properties: {
							left: {
								start: 0,
								end: thisPos.w
							}
						},
						onEnd: function() {
							deferred.callback(tailWidget);
						}
					}).play();
				}

				var len = this._widgetStack.length;
				if (len) {
					var newTailWidget = this._widgetStack[len - 1];
					newTailWidget.domNode.style.display = ''; //$NON-NLS-1$

					var inAnim = dojo.animateProperty({
						node: newTailWidget.domNode,
						duration: SLIDE_DURATION,
						easing: EASING,
						properties: {
							left: {
								end: 0
							}
						}
					}).play();
				}

				return deferred;
			},

			postCreate: function() {
				this.inherited(arguments);

				dojo.style(this.domNode, {
					overflow: 'hidden', //$NON-NLS-1$
					position: 'relative' //$NON-NLS-1$
				});
			},

			uninitialize: function() {
				for (var i = 0; i < this._widgetStack.length; i++) {
					var widget = this._widgetStack[i];
					widget.destroyRecursive();
				}

				this.inherited(arguments);
			}
		}
	);

})();