/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2012. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide('com.ibm.relm.ple.web.ui.tree.PinnedModeButton'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.tree', 'PinnedModeButtonMessages'); //$NON-NLS-1$ //$NON-NLS-2$
dojo.require('dijit._Templated'); //$NON-NLS-1$

(function(){
	var messages = dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.tree', 'PinnedModeButtonMessages'); //$NON-NLS-1$ //$NON-NLS-2$

	/**
	 * The actual menu listing the known pinned mode types. When an item is clicked
	 * it passed back the new pinned mode to its button. This is designed to be used
	 * from the PinnedModeButton only.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.tree.PinnedModeMenu', //$NON-NLS-1$
		[ jazz.ui.Menu ],
		{
			
			selectable: true,

			/**
			 * The parent PinnedModeButton that this menu will call back to.
			 */
			button: null,

			/**
			 * The current pinned mode string. Setting this updated the
			 * selected menu item.
			 * 
			 * @type {string}
			 */
			pinnedMode: null,

			constructor: function(args) {
				var button = args.button;

				// We actually create the menu item here instead of postCreate
				// so that the construcotr can take a `pinnedMode` property
				// and be able to set it internally before postCreate is called,
				// which requires these menu items already exist.
				// We'll still add them to the menu in postCreate.
				this._unpinnedMenuItem = new jazz.ui.menu.MenuItem({
					iconClass: 'ple-icon-unpinned', //$NON-NLS-1$
					label: messages.unpinned,
					onClick: dojo.hitch(button, button._onPinnedModeClicked, null)	// passes `null` as the value of unpinned.
				});	

				this._pinVersionMenuItem = new jazz.ui.menu.MenuItem({
					iconClass: 'ple-icon-pinned', //$NON-NLS-1$
					label: messages.pinVersion,
					onClick: dojo.hitch(button, button._onPinnedModeClicked, 'versionPinned') //$NON-NLS-1$
				});

				this._pinBranchMenuItem = new jazz.ui.menu.MenuItem({
					iconClass: 'ple-icon-branchPinned', //$NON-NLS-1$
					label: messages.pinBranch,
					onClick: dojo.hitch(button, button._onPinnedModeClicked, 'branchPinned') //$NON-NLS-1$
				});
			},

			postCreate: function() {
				this.inherited(arguments);

				this.addChild(this._unpinnedMenuItem);
				this.addChild(this._pinVersionMenuItem);
				this.addChild(this._pinBranchMenuItem);
			},

			_setPinnedModeAttr: function(pinnedMode) {
				this._set('pinnedMode', pinnedMode); //$NON-NLS-1$

				this._unpinnedMenuItem.set('selected', !pinnedMode); //$NON-NLS-1$
				this._pinVersionMenuItem.set('selected', pinnedMode === 'versionPinned'); //$NON-NLS-1$ //$NON-NLS-2$
				this._pinBranchMenuItem.set('selected', pinnedMode === 'branchPinned' || pinnedMode === 'branchPinnedWhileMutable'); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
			}
		}
	);
	
	/**
	 * Given a pinned mode, returns the class name for its related icon
	 * and a tooltip describing it.
	 * @param  {string} pinnedMode pinned mode or null if unpinned.
	 * @return {array}            two item array of class name and a tooltip.
	 */
	function getIconClassAndTooltipForPinnedMode(pinnedMode) {
		if (pinnedMode === 'versionPinned') { //$NON-NLS-1$
			return ['ple-icon-pinned', messages.versionIsPinned]; //$NON-NLS-1$
		}
		else if (pinnedMode === 'branchPinned' || pinnedMode === 'branchPinnedWhileMutable') { //$NON-NLS-1$ //$NON-NLS-2$
			return ['ple-icon-branchPinned', messages.branchIsPinned]; //$NON-NLS-1$
		}
		else {
			return ['ple-icon-unpinned', messages.unpinned]; //$NON-NLS-1$
		}
	}

	/**
	 * A button that shows and sets the pinned mode of an item version. This
	 * is used directly in the the product tree toolbar. The button will set a
	 * class to make itself always visible if it has any interesting pinned
	 * mode.
	 */
	dojo.declare(
		'com.ibm.relm.ple.web.ui.tree.PinnedModeButton', //$NON-NLS-1$
		[ jazz.ui.toolbar.MenuButton ],
		{
			destroyOnClose: true,
			injectCaret: false,
			
			/**
			 * Called back when the pinned mode changes.
			 * Set in constructor or later. Sent 1 argument: the new pinned mode.
			 */
			onChange: null,

			/**
			 * The pinned mode of the button. Can be set at construction or
			 * later. Will update the button icon and selected menu item to
			 * match the current pinned mode.
			 * 
			 * @type {string} the pinned mode name or null is unpinned. Modes can
			 * be `versionPinned`, `branchPinned` and `branchPinnedWhileMutable`. 
			 * Note that the two branch modes look identical in the UI.
			 */
			pinnedMode: null,

			constructor: function() {
				this.menuProvider = dojo.hitch(this, this._createMenu);
			},

			_setPinnedModeAttr: function(pinnedMode) {
				this._set('pinnedMode', pinnedMode); //$NON-NLS-1$

				dojo.toggleClass(this.domNode, 'ple-OnOffButton-on', !!pinnedMode); //$NON-NLS-1$

				var iconClassAndTitle = getIconClassAndTooltipForPinnedMode(pinnedMode);
				this.set('imgClass', iconClassAndTitle[0]); //$NON-NLS-1$
				this.set('title', iconClassAndTitle[1]); //$NON-NLS-1$
			},

			/**
			 * Menu provider function to create a new menu with the proper
			 * pinned mode selected.
			 * @return {jazz.ui.Menu}
			 */
			_createMenu: function() {
				var menu = new com.ibm.relm.ple.web.ui.tree.PinnedModeMenu({
					button: this,
					pinnedMode: this.pinnedMode
				});
				menu.startup();
				return menu;
			},

			/**
			 * Called by the menu items when the pinned mode is changed. This
			 * doesn't update the actual pinned mode since that usually
			 * requires a server call and the new visual pinned mode can only
			 * be seen after refreshing the tree.
			 * 
			 * @param  {string} pinnedMode the new pinned mode.
			 */
			_onPinnedModeClicked: function(pinnedMode) {
				// Avoid callbacks if nothing changed
				// Use "==" to ensure null==undefined in this case.
				if (this.pinnedMode == pinnedMode) return;

				// Only fire this after .startup() so that the initial state
				// can be set without firing the callback
				if (this.onChange && this._started) this.onChange(pinnedMode);
			}
		}
	);
	
	/**
	 * Gets a class name of the icon to show for a given pinned mode.
	 * 
	 * @type {string} the name of the pinned mode (as saved on JSON items).
	 * @return {string} name of the icon class to use.
	 */
	com.ibm.relm.ple.web.ui.tree.PinnedModeButton.getIconClassForPinnedMode = function(pinnedMode) {
		getIconClassAndTooltipForPinnedMode(pinnedMode)[0];
	};
})();
