dojo.provide('com.ibm.relm.ple.web.ui.tree.TreeRefresh'); //$NON-NLS-1$

dojo.require('com.ibm.relm.ple.web.ui.Goto'); //$NON-NLS-1$
dojo.require("net.jazz.ajax.ui.PlatformUI"); //$NON-NLS-1$
dojo.require('com.ibm.relm.common.web.StatusMessage'); //$NON-NLS-1$

dojo.require('dojo.i18n'); //$NON-NLS-1$
dojo.requireLocalization('com.ibm.relm.ple.web.ui.tree', 'TreeRefreshMessages'); //$NON-NLS-1$ //$NON-NLS-2$

/**
 * Simplifies refreshing the tree and selecting the proper node afterwards.
 * Exposes static functions that decouple calling code from the exact
 * implementation of the product tree (which has changed several times) and
 * provides and simple interface for refreshing.
 */
(function(){

var Goto = com.ibm.relm.ple.web.ui.Goto;
var StatusMessage = com.ibm.relm.common.web.StatusMessage;

/**
 * Returns the parent path up to the last path component.
 * @param  {array} path the path
 * @return {array} parent path.
 */
function parentPath(path) {
	return path.slice(0, path.length - 1);
}

/**
 * The single known product tree. This gets reset by the sidebar after a full refresh,
 * but is generally always set.
 */
var tree_ = null;

var messages_ = dojo.i18n.getLocalization('com.ibm.relm.ple.web.ui.tree', 'TreeRefreshMessages'); //$NON-NLS-1$ //$NON-NLS-2$

dojo.mixin(com.ibm.relm.ple.web.ui.tree.TreeRefresh, {

	setTree: function(newTree) {
		tree_ = newTree;
	},

	/**
	 * Gets the last set tree, which should be the visible product tree except
	 * during a hard refresh when it's not yet set. This feels like a bit of a
	 * hack exposing this here and was only added as a convenience for drag
	 * and drop.
	 * 
	 * @return {AltProductTree} the current product tree or null if not set
	 *     yet.
	 */
	getTree: function() {
		return tree_;
	},

	/**
	 * Refreshes the product tree. All visible nodes will be updated with the
	 * latest changes. Collapsed nodes may be uncached and will need to lazy
	 * load again. See `args` for operating on the refreshed tree.
	 * 	
	 * @param  {object} args Details below:
	 * 
	 * args.selectPath {path array} - to open the tree and select a product after the
	 *     refresh. A path is an array of path segments as found on the sites
	 *     returned from the TreeSelection class. If a path is selected, it
	 *     will automatically cause the detail pane to be updated with the
	 *     current version of the selected path even if the path itself didn't
	 *     change since the last selection.
	 *   
	 * args.selectParentPath {path array} - is like selectPath, but the parent (one up from the
	 *     leaf of the path) will be selected. This is used when adding links,
	 *     which we don't want to select--we select the parent product. This
	 *     is mutually exclusive with args.selectPath.
	 *     
	 * args.flashPath {boolean} - true to visually flash the selected node provided in
	 *     .selectPath, for example, after a product create. If
	 *     args.selectParentPath is used, then the leaf node of the path
	 *     (usually a link) will still flash, even though the parent is what
	 *     is selected.
	 *     
	 * args.openPath {boolean} - true to open the selected node (normally only
	 *     its parents are opened). That is, arg.selectPath will select that
	 *     path in the tree, but if the selected node was not already open, it
	 *     will be left closed. By also specifying this, that node will be
	 *     opened so that its children are visible, even though the children
	 *     aren't selected. Mutually exclusive with flashPath.
	 *
	 * args.noGoto {boolean} - if true, will not change the browser URL to
	 * 		select the given path. This is basically a work around for the
	 * 		Manage Dimensions page, which wants to refresh the tree, needs to
	 * 		potentially refresh the focused product, but does not want to
	 * 		switch to the tree view. I (Jay) suspect this is not quite right
	 * 		and may not work if the focused product become unbound. Needs
	 * 		testing.
	 *
	 * args.clearPath {boolean} - true to clear the path after refresh, that is, nothing will be
	 * selected. Used after removing tree nodes.
	 * 
	 * @return {dojo.Deferred} callbacks will fire after the tree is
	 *     refreshed. Callers usually should use the know when to hide
	 *     progress indicators or close a dialog.
	 */
	refresh: function(args) {
		dojo.publish("ple.product_tree_hide_message"); //$NON-NLS-1$
		
		args = args || {};
		
		if (!tree_) {
			console.info('Product tree doesn\'t exist to refresh'); //$NON-NLS-1$
			var fakeDeferred = new dojo.Deferred();
			fakeDeferred.callback();
			return fakeDeferred;
		}

		var refreshingMessage = StatusMessage.show(messages_.refreshingProducts);

		var deferredRefresh = tree_.refresh();

		var eitherPath = args.selectParentPath || args.selectPath;

		if (eitherPath) {
			var pathToSelect = args.selectParentPath ? parentPath(args.selectParentPath) : args.selectPath;

			deferredRefresh.then(function() {
				tree_.openPath(eitherPath).then(selectAndFlash).otherwise(function(errorMessage) {
					console.warn('Failed to select the new path in the product tree: ' + errorMessage + '. Will attempt to load right now...'); //$NON-NLS-1$ //$NON-NLS-2$
					selectAndFlash(null);
				});
			});
		}
		else if (args.clearPath) {
			deferredRefresh.then(function() {
				dojo.publish('ple.select_nothing'); //$NON-NLS-1$
			});
		}

		function selectAndFlash(selectedTreeNode) {
			if ((args.noGoto || !Goto.gotoProductTreePath(pathToSelect)) && selectedTreeNode) {
				// If the location already matches the path we get here. No
				// hashchange event fires, so the details view isn't
				// automatically reloaded. Force it here if requested.
				var selectedItem = args.selectParentPath
					? selectedTreeNode.parentTreeNode().item
					: selectedTreeNode.item;
				dojo.publish('ple.load_any_single_item_details', [selectedItem]); //$NON-NLS-1$
			}

			if (args.openPath) {
				var pathToOpen = eitherPath;
				tree_.getNodeAtPath(pathToOpen).open();
			}
			else if (args.flashPath) {
				var flashArgs = {
					// Match the selected row color when flashing new
					// node. If selecting parent, use unselected row
					// color.
					endColor: args.selectParentPath ? '#fbfbfb' : '#ffeacf' //$NON-NLS-1$ //$NON-NLS-2$
				};

				var pathToFlash = eitherPath;
				tree_.flashNodeAtPath(pathToFlash, flashArgs);
			}
		}

		deferredRefresh.addBoth(function() {
			refreshingMessage.hide();
		});

		return deferredRefresh;
	}
});

})();